@@ -19,20 +19,22 @@
 /*! \file
  *
  * \brief Dial plan macro Implementation
+ *
+ * \author Mark Spencer <markster@digium.com>
  * 
  * \ingroup applications
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 77768 $")
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 66744 $")
-
 #include "asterisk/file.h"
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
@@ -48,8 +50,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 66744 $")
 /* special result value used to force macro exit */
 #define MACRO_EXIT_RESULT 1024
 
-static char *tdesc = "Extension Macros";
-
 static char *descrip =
 "  Macro(macroname|arg1|arg2...): Executes a macro using the context\n"
 "'macro-<macroname>', jumping to the 's' extension of that context and\n"
@@ -70,8 +70,8 @@ static char *descrip =
 "         contained within it via sub-engine), and a fixed per-thread\n"
 "         memory stack allowance, macros are limited to 7 levels\n"
 "         of nesting (macro calling macro calling macro, etc.); It\n"
-"         may be possible that stack-intensive applications in deeply nested\n"
-"         macros could cause asterisk to crash earlier than this limit.\n";
+"         may be possible that stack-intensive applications in deeply nested macros\n"
+"         could cause asterisk to crash earlier than this limit.\n";
 
 static char *if_descrip =
 "  MacroIf(<expr>?macroname_a[|arg1][:macroname_b[|arg1]])\n"
@@ -79,6 +79,13 @@ static char *if_descrip =
 "(otherwise <macroname_b> if provided)\n"
 "Arguments and return values as in application macro()\n";
 
+static char *exclusive_descrip =
+"  MacroExclusive(macroname|arg1|arg2...):\n"
+"Executes macro defined in the context 'macro-macroname'\n"
+"Only one call at a time may run the macro.\n"
+"(we'll wait if another call is busy executing in the Macro)\n"
+"Arguments and return values as in application Macro()\n";
+
 static char *exit_descrip =
 "  MacroExit():\n"
 "Causes the currently running macro to exit as if it had\n"
@@ -88,15 +95,14 @@ static char *exit_descrip =
 
 static char *app = "Macro";
 static char *if_app = "MacroIf";
+static char *exclusive_app = "MacroExclusive";
 static char *exit_app = "MacroExit";
 
 static char *synopsis = "Macro Implementation";
 static char *if_synopsis = "Conditional Macro Implementation";
+static char *exclusive_synopsis = "Exclusive Macro Implementation";
 static char *exit_synopsis = "Exit From Macro";
 
-STANDARD_LOCAL_USER;
-
-LOCAL_USER_DECL;
 
 static struct ast_exten *find_matching_priority(struct ast_context *c, const char *exten, int priority, const char *callerid)
 {
@@ -133,8 +139,9 @@ static struct ast_exten *find_matching_priority(struct ast_context *c, const cha
 	return NULL;
 }
 
-static int macro_exec(struct ast_channel *chan, void *data)
+static int _macro_exec(struct ast_channel *chan, void *data, int exclusive)
 {
+	const char *s;
 	char *tmp;
 	char *cur, *rest;
 	char *macro;
@@ -148,7 +155,7 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	int oldpriority, gosub_level = 0;
 	char pc[80], depthc[12];
 	char oldcontext[AST_MAX_CONTEXT] = "";
-	char *offsets, *s, *inhangupc;
+	const char *inhangupc;
 	int offset, depth = 0, maxdepth = 7;
 	int setmacrocontext=0;
 	int autoloopflag, dead = 0, inhangup = 0;
@@ -157,14 +164,14 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	char *save_macro_context;
 	char *save_macro_priority;
 	char *save_macro_offset;
-	struct localuser *u;
+	struct ast_module_user *u;
  
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "Macro() requires arguments. See \"show application macro\" for help.\n");
 		return -1;
 	}
 
-	LOCAL_USER_ADD(u);
+	u = ast_module_user_add(chan);
 
 	/* does the user want a deeper rabbit hole? */
 	s = pbx_builtin_getvar_helper(chan, "MACRO_RECURSION");
@@ -172,13 +179,9 @@ static int macro_exec(struct ast_channel *chan, void *data)
 		sscanf(s, "%d", &maxdepth);
 
 	/* Count how many levels deep the rabbit hole goes */
-	tmp = pbx_builtin_getvar_helper(chan, "MACRO_DEPTH");
-	if (tmp) {
-		sscanf(tmp, "%d", &depth);
-	} else {
-		depth = 0;
-	}
-
+	s = pbx_builtin_getvar_helper(chan, "MACRO_DEPTH");
+	if (s)
+		sscanf(s, "%d", &depth);
 	/* Used for detecting whether to return when a Macro is called from another Macro after hangup */
 	if (strcmp(chan->exten, "h") == 0)
 		pbx_builtin_setvar_helper(chan, "MACRO_IN_HANGUP", "1");
@@ -188,7 +191,7 @@ static int macro_exec(struct ast_channel *chan, void *data)
 
 	if (depth >= maxdepth) {
 		ast_log(LOG_ERROR, "Macro():  possible infinite loop detected.  Returning early.\n");
-		LOCAL_USER_REMOVE(u);
+		ast_module_user_remove(u);
 		return 0;
 	}
 	snprintf(depthc, sizeof(depthc), "%d", depth + 1);
@@ -199,18 +202,33 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	macro = strsep(&rest, "|");
 	if (ast_strlen_zero(macro)) {
 		ast_log(LOG_WARNING, "Invalid macro name specified\n");
-		LOCAL_USER_REMOVE(u);
+		ast_module_user_remove(u);
 		return 0;
 	}
+
 	snprintf(fullmacro, sizeof(fullmacro), "macro-%s", macro);
 	if (!ast_exists_extension(chan, fullmacro, "s", 1, chan->cid.cid_num)) {
-  		if (!ast_context_find(fullmacro)) 
+		if (!ast_context_find(fullmacro)) 
 			ast_log(LOG_WARNING, "No such context '%s' for macro '%s'\n", fullmacro, macro);
 		else
-	  		ast_log(LOG_WARNING, "Context '%s' for macro '%s' lacks 's' extension, priority 1\n", fullmacro, macro);
-		LOCAL_USER_REMOVE(u);
+			ast_log(LOG_WARNING, "Context '%s' for macro '%s' lacks 's' extension, priority 1\n", fullmacro, macro);
+		ast_module_user_remove(u);
 		return 0;
 	}
+
+	/* If we are to run the macro exclusively, take the mutex */
+	if (exclusive) {
+		ast_log(LOG_DEBUG, "Locking macrolock for '%s'\n", fullmacro);
+		ast_autoservice_start(chan);
+		if (ast_context_lockmacro(fullmacro)) {
+			ast_log(LOG_WARNING, "Failed to lock macro '%s' as in-use\n", fullmacro);
+			ast_autoservice_stop(chan);
+			ast_module_user_remove(u);
+
+			return 0;
+		}
+		ast_autoservice_stop(chan);
+	}
 	
 	/* Save old info */
 	oldpriority = chan->priority;
@@ -224,25 +242,17 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	}
 	argc = 1;
 	/* Save old macro variables */
-	save_macro_exten = pbx_builtin_getvar_helper(chan, "MACRO_EXTEN");
-	if (save_macro_exten) 
-		save_macro_exten = strdup(save_macro_exten);
+	save_macro_exten = ast_strdup(pbx_builtin_getvar_helper(chan, "MACRO_EXTEN"));
 	pbx_builtin_setvar_helper(chan, "MACRO_EXTEN", oldexten);
 
-	save_macro_context = pbx_builtin_getvar_helper(chan, "MACRO_CONTEXT");
-	if (save_macro_context)
-		save_macro_context = strdup(save_macro_context);
+	save_macro_context = ast_strdup(pbx_builtin_getvar_helper(chan, "MACRO_CONTEXT"));
 	pbx_builtin_setvar_helper(chan, "MACRO_CONTEXT", oldcontext);
 
-	save_macro_priority = pbx_builtin_getvar_helper(chan, "MACRO_PRIORITY");
-	if (save_macro_priority) 
-		save_macro_priority = strdup(save_macro_priority);
+	save_macro_priority = ast_strdup(pbx_builtin_getvar_helper(chan, "MACRO_PRIORITY"));
 	snprintf(pc, sizeof(pc), "%d", oldpriority);
 	pbx_builtin_setvar_helper(chan, "MACRO_PRIORITY", pc);
   
-	save_macro_offset = pbx_builtin_getvar_helper(chan, "MACRO_OFFSET");
-	if (save_macro_offset) 
-		save_macro_offset = strdup(save_macro_offset);
+	save_macro_offset = ast_strdup(pbx_builtin_getvar_helper(chan, "MACRO_OFFSET"));
 	pbx_builtin_setvar_helper(chan, "MACRO_OFFSET", NULL);
 
 	/* Setup environment for new run */
@@ -252,12 +262,13 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	chan->priority = 1;
 
 	while((cur = strsep(&rest, "|")) && (argc < MAX_ARGS)) {
+		const char *s;
   		/* Save copy of old arguments if we're overwriting some, otherwise
 	   	let them pass through to the other macro */
   		snprintf(varname, sizeof(varname), "ARG%d", argc);
-		oldargs[argc] = pbx_builtin_getvar_helper(chan, varname);
-		if (oldargs[argc])
-			oldargs[argc] = strdup(oldargs[argc]);
+		s = pbx_builtin_getvar_helper(chan, varname);
+		if (s)
+			oldargs[argc] = ast_strdup(s);
 		pbx_builtin_setvar_helper(chan, varname, cur);
 		argc++;
 	}
@@ -309,14 +320,14 @@ static int macro_exec(struct ast_channel *chan, void *data)
 			case AST_PBX_KEEPALIVE:
 				if (option_debug)
 					ast_log(LOG_DEBUG, "Spawn extension (%s,%s,%d) exited KEEPALIVE in macro %s on '%s'\n", chan->context, chan->exten, chan->priority, macro, chan->name);
-				if (option_verbose > 1)
+				else if (option_verbose > 1)
 					ast_verbose( VERBOSE_PREFIX_2 "Spawn extension (%s, %s, %d) exited KEEPALIVE in macro '%s' on '%s'\n", chan->context, chan->exten, chan->priority, macro, chan->name);
 				goto out;
 				break;
 			default:
 				if (option_debug)
 					ast_log(LOG_DEBUG, "Spawn extension (%s,%s,%d) exited non-zero on '%s' in macro '%s'\n", chan->context, chan->exten, chan->priority, chan->name, macro);
-				if (option_verbose > 1)
+				else if (option_verbose > 1)
 					ast_verbose( VERBOSE_PREFIX_2 "Spawn extension (%s, %s, %d) exited non-zero on '%s' in macro '%s'\n", chan->context, chan->exten, chan->priority, chan->name, macro);
 				dead = 1;
 				goto out;
@@ -397,7 +408,6 @@ static int macro_exec(struct ast_channel *chan, void *data)
 	snprintf(depthc, sizeof(depthc), "%d", depth);
 	if (!dead) {
 		pbx_builtin_setvar_helper(chan, "MACRO_DEPTH", depthc);
-
 		ast_set2_flag(chan, autoloopflag, AST_FLAG_IN_AUTOLOOP);
 	}
 
@@ -438,6 +448,7 @@ static int macro_exec(struct ast_channel *chan, void *data)
 		ast_copy_string(chan->context, oldcontext, sizeof(chan->context));
 		if (!(chan->_softhangup & AST_SOFTHANGUP_ASYNCGOTO)) {
 			/* Copy the extension, so long as we're not in softhangup, where we could be given an asyncgoto */
+			const char *offsets;
 			ast_copy_string(chan->exten, oldexten, sizeof(chan->exten));
 			if ((offsets = pbx_builtin_getvar_helper(chan, "MACRO_OFFSET"))) {
 				/* Handle macro offset if it's set by checking the availability of step n + offset + 1, otherwise continue
@@ -455,22 +466,41 @@ static int macro_exec(struct ast_channel *chan, void *data)
 		pbx_builtin_setvar_helper(chan, "MACRO_OFFSET", save_macro_offset);
 	if (save_macro_offset)
 		free(save_macro_offset);
-	LOCAL_USER_REMOVE(u);
+
+	/* Unlock the macro */
+	if (exclusive) {
+		ast_log(LOG_DEBUG, "Unlocking macrolock for '%s'\n", fullmacro);
+		if (ast_context_unlockmacro(fullmacro)) {
+			ast_log(LOG_ERROR, "Failed to unlock macro '%s' - that isn't good\n", fullmacro);
+			res = 0;
+		}
+	}
+	
+	ast_module_user_remove(u);
+
 	return res;
 }
 
+static int macro_exec(struct ast_channel *chan, void *data)
+{
+	return _macro_exec(chan, data, 0);
+}
+
+static int macroexclusive_exec(struct ast_channel *chan, void *data)
+{
+	return _macro_exec(chan, data, 1);
+}
+
 static int macroif_exec(struct ast_channel *chan, void *data) 
 {
 	char *expr = NULL, *label_a = NULL, *label_b = NULL;
 	int res = 0;
-	struct localuser *u;
+	struct ast_module_user *u;
 
-	LOCAL_USER_ADD(u);
+	u = ast_module_user_add(chan);
 
-	expr = ast_strdupa(data);
-	if (!expr) {
-		ast_log(LOG_ERROR, "Out of Memory!\n");
-		LOCAL_USER_REMOVE(u);
+	if (!(expr = ast_strdupa(data))) {
+		ast_module_user_remove(u);
 		return -1;
 	}
 
@@ -482,13 +512,13 @@ static int macroif_exec(struct ast_channel *chan, void *data)
 			label_b++;
 		}
 		if (pbx_checkcondition(expr))
-			macro_exec(chan, label_a);
+			res = macro_exec(chan, label_a);
 		else if (label_b) 
-			macro_exec(chan, label_b);
+			res = macro_exec(chan, label_b);
 	} else
 		ast_log(LOG_WARNING, "Invalid Syntax.\n");
 
-	LOCAL_USER_REMOVE(u);
+	ast_module_user_remove(u);
 
 	return res;
 }
@@ -498,43 +528,30 @@ static int macro_exit_exec(struct ast_channel *chan, void *data)
 	return MACRO_EXIT_RESULT;
 }
 
-int unload_module(void)
+static int unload_module(void)
 {
 	int res;
 
 	res = ast_unregister_application(if_app);
 	res |= ast_unregister_application(exit_app);
 	res |= ast_unregister_application(app);
+	res |= ast_unregister_application(exclusive_app);
 
-	STANDARD_HANGUP_LOCALUSERS;
+	ast_module_user_hangup_all();
 
 	return res;
 }
 
-int load_module(void)
+static int load_module(void)
 {
 	int res;
 
 	res = ast_register_application(exit_app, macro_exit_exec, exit_synopsis, exit_descrip);
 	res |= ast_register_application(if_app, macroif_exec, if_synopsis, if_descrip);
+	res |= ast_register_application(exclusive_app, macroexclusive_exec, exclusive_synopsis, exclusive_descrip);
 	res |= ast_register_application(app, macro_exec, synopsis, descrip);
 
 	return res;
 }
 
-char *description(void)
-{
-	return tdesc;
-}
-
-int usecount(void)
-{
-	int res;
-	STANDARD_USECOUNT(res);
-	return res;
-}
-
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Extension Macros");
