@@ -2,7 +2,7 @@
  * Asterisk -- An open source telephony toolkit.
  *
  * Copyright (C) 2005, Anthony Minessale II
- * Copyright (C) 2005, Digium, Inc.
+ * Copyright (C) 2005 - 2006, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  * Kevin P. Fleming <kpfleming@digium.com>
@@ -22,19 +22,26 @@
  */
 
 /*! \file
+ *
  * \brief MixMonitor() - Record a call and mix the audio during the recording
  * \ingroup applications
+ *
+ * \author Mark Spencer <markster@digium.com>
+ * \author Kevin P. Fleming <kpfleming@digium.com>
+ *
+ * \note Based on app_muxmon.c provided by
+ * Anthony Minessale II <anthmct@yahoo.com>
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 72381 $")
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 72378 $")
-
 #include "asterisk/file.h"
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
@@ -46,10 +53,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 72378 $")
 #include "asterisk/options.h"
 #include "asterisk/app.h"
 #include "asterisk/linkedlists.h"
+#include "asterisk/utils.h"
 
 #define get_volfactor(x) x ? ((x > 0) ? (1 << x) : ((1 << abs(x)) * -1)) : 0
 
-static const char *tdesc = "Mixed Audio Monitoring Application";
 static const char *app = "MixMonitor";
 static const char *synopsis = "Record a call and mix the audio during the recording";
 static const char *desc = ""
@@ -73,9 +80,15 @@ static const char *desc = ""
 "The variable MIXMONITOR_FILENAME will contain the filename used to record.\n"
 "";
 
-STANDARD_LOCAL_USER;
+static const char *stop_app = "StopMixMonitor";
+static const char *stop_synopsis = "Stop recording a call through MixMonitor";
+static const char *stop_desc = ""
+"  StopMixMonitor()\n\n"
+"Stops the audio recording that was started with a call to MixMonitor()\n"
+"on the current channel.\n"
+"";
 
-LOCAL_USER_DECL;
+struct module_symbols *me;
 
 static const char *mixmonitor_spy_type = "MixMonitor";
 
@@ -118,9 +131,9 @@ static int startmon(struct ast_channel *chan, struct ast_channel_spy *spy)
 	if (!chan)
 		return -1;
 
-	ast_mutex_lock(&chan->lock);
+	ast_channel_lock(chan);
 	res = ast_channel_spy_add(chan, spy);
-	ast_mutex_unlock(&chan->lock);
+	ast_channel_unlock(chan);
 
 	if (!res && ast_test_flag(chan, AST_FLAG_NBRIDGE) && (peer = ast_bridged_channel(chan)))
 		ast_softhangup(peer, AST_SOFTHANGUP_UNBRIDGE);	
@@ -138,10 +151,7 @@ static void *mixmonitor_thread(void *obj)
 	unsigned int oflags;
 	char *ext;
 	int errflag = 0;
-	
-	
-	STANDARD_INCREMENT_USECOUNT;
-	
+
 	if (option_verbose > 1)
 		ast_verbose(VERBOSE_PREFIX_2 "Begin MixMonitor Recording %s\n", mixmonitor->name);
 	
@@ -167,7 +177,7 @@ static void *mixmonitor_thread(void *obj)
 			   of frames if a queue flush was necessary, so process them
 			*/
 			for (; f; f = next) {
-				next = f->next;
+				next = AST_LIST_NEXT(f, frame_list);
 				if (write && errflag == 0) {
 					if (!fs) {
 						/* Determine creation flags and filename plus extension for filestream */
@@ -189,15 +199,15 @@ static void *mixmonitor_thread(void *obj)
 					if (fs)
 						ast_writestream(fs, f);
 				}
-				ast_frfree(f);
+				ast_frame_free(f, 0);
 			}
 		}
 	}
 
 	ast_mutex_unlock(&mixmonitor->spy.lock);
-	
-	ast_channel_spy_free(&mixmonitor->spy);
 
+	ast_channel_spy_free(&mixmonitor->spy);
+	
 	if (option_verbose > 1)
 		ast_verbose(VERBOSE_PREFIX_2 "End MixMonitor Recording %s\n", mixmonitor->name);
 
@@ -212,7 +222,6 @@ static void *mixmonitor_thread(void *obj)
 
 	free(mixmonitor);
 
-	STANDARD_DECREMENT_USECOUNT;
 
 	return NULL;
 }
@@ -246,7 +255,6 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 
 	/* Pre-allocate mixmonitor structure and spy */
 	if (!(mixmonitor = calloc(1, len))) {
-		ast_log(LOG_ERROR, "Memory Error!\n");
 		return;
 	}
 
@@ -290,7 +298,7 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	ast_pthread_create(&thread, &attr, mixmonitor_thread, mixmonitor);
+	ast_pthread_create_background(&thread, &attr, mixmonitor_thread, mixmonitor);
 	pthread_attr_destroy(&attr);
 
 }
@@ -298,7 +306,7 @@ static void launch_monitor_thread(struct ast_channel *chan, const char *filename
 static int mixmonitor_exec(struct ast_channel *chan, void *data)
 {
 	int x, readvol = 0, writevol = 0;
-	struct localuser *u;
+	struct ast_module_user *u;
 	struct ast_flags flags = {0};
 	char *parse;
 	AST_DECLARE_APP_ARGS(args,
@@ -312,19 +320,15 @@ static int mixmonitor_exec(struct ast_channel *chan, void *data)
 		return -1;
 	}
 
-	LOCAL_USER_ADD(u);
+	u = ast_module_user_add(chan);
 
-	if (!(parse = ast_strdupa(data))) {
-		ast_log(LOG_WARNING, "Memory Error!\n");
-		LOCAL_USER_REMOVE(u);
-		return -1;
-	}
+	parse = ast_strdupa(data);
 
 	AST_STANDARD_APP_ARGS(args, parse);
 	
 	if (ast_strlen_zero(args.filename)) {
 		ast_log(LOG_WARNING, "MixMonitor requires an argument (filename)\n");
-		LOCAL_USER_REMOVE(u);
+		ast_module_user_remove(u);
 		return -1;
 	}
 
@@ -376,7 +380,22 @@ static int mixmonitor_exec(struct ast_channel *chan, void *data)
 	pbx_builtin_setvar_helper(chan, "MIXMONITOR_FILENAME", args.filename);
 	launch_monitor_thread(chan, args.filename, flags.flags, readvol, writevol, args.post_process);
 
-	LOCAL_USER_REMOVE(u);
+	ast_module_user_remove(u);
+
+	return 0;
+}
+
+static int stop_mixmonitor_exec(struct ast_channel *chan, void *data)
+{
+	struct ast_module_user *u;
+
+	u = ast_module_user_add(chan);
+
+	ast_channel_lock(chan);
+	ast_channel_spy_stop_by_type(chan, mixmonitor_spy_type);
+	ast_channel_unlock(chan);
+
+	ast_module_user_remove(u);
 
 	return 0;
 }
@@ -398,57 +417,47 @@ static int mixmonitor_cli(int fd, int argc, char **argv)
 	else if (!strcasecmp(argv[1], "stop"))
 		ast_channel_spy_stop_by_type(chan, mixmonitor_spy_type);
 
-	ast_mutex_unlock(&chan->lock);
+	ast_channel_unlock(chan);
 
 	return RESULT_SUCCESS;
 }
 
+static char *complete_mixmonitor_cli(const char *line, const char *word, int pos, int state)
+{
+	return ast_complete_channels(line, word, pos, state, 2);
+}
 
-static struct ast_cli_entry cli_mixmonitor = {
-	{ "mixmonitor", NULL, NULL },
-	mixmonitor_cli, 
-	"Execute a MixMonitor command",
-	"mixmonitor <start|stop> <chan_name> [<args>]\n"
+static struct ast_cli_entry cli_mixmonitor[] = {
+	{ { "mixmonitor", NULL, NULL },
+	mixmonitor_cli, "Execute a MixMonitor command.",
+	"mixmonitor <start|stop> <chan_name> [args]\n\n"
+	"The optional arguments are passed to the\n"
+	"MixMonitor application when the 'start' command is used.\n",
+	complete_mixmonitor_cli },
 };
 
-
-int unload_module(void)
+static int unload_module(void)
 {
 	int res;
 
-	res = ast_cli_unregister(&cli_mixmonitor);
+	ast_cli_unregister_multiple(cli_mixmonitor, sizeof(cli_mixmonitor) / sizeof(struct ast_cli_entry));
+	res = ast_unregister_application(stop_app);
 	res |= ast_unregister_application(app);
 	
-	STANDARD_HANGUP_LOCALUSERS;
+	ast_module_user_hangup_all();
 
 	return res;
 }
 
-int load_module(void)
+static int load_module(void)
 {
 	int res;
 
-	res = ast_cli_register(&cli_mixmonitor);
-	res |= ast_register_application(app, mixmonitor_exec, synopsis, desc);
+	ast_cli_register_multiple(cli_mixmonitor, sizeof(cli_mixmonitor) / sizeof(struct ast_cli_entry));
+	res = ast_register_application(app, mixmonitor_exec, synopsis, desc);
+	res |= ast_register_application(stop_app, stop_mixmonitor_exec, stop_synopsis, stop_desc);
 
 	return res;
 }
 
-char *description(void)
-{
-	return (char *) tdesc;
-}
-
-int usecount(void)
-{
-	int res;
-
-	STANDARD_USECOUNT(res);
-
-	return res;
-}
-
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Mixed Audio Monitoring Application");
