@@ -21,11 +21,17 @@
 /*! \file
  *
  * \brief Comma Separated Value CDR records.
+ *
+ * \author Mark Spencer <markster@digium.com>
  * 
  * \arg See also \ref AstCDR
  * \ingroup cdr_drivers
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 69392 $")
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
@@ -35,10 +41,7 @@
 #include <unistd.h>
 #include <time.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
-
+#include "asterisk/config.h"
 #include "asterisk/channel.h"
 #include "asterisk/cdr.h"
 #include "asterisk/module.h"
@@ -50,6 +53,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
 
 #define DATE_FORMAT "%Y-%m-%d %T"
 
+static int usegmtime = 0;
+static int loguniqueid = 0;
+static int loguserfield = 0;
+static char *config = "cdr.conf";
+
 /* #define CSV_LOGUNIQUEID 1 */
 /* #define CSV_LOGUSERFIELD 1 */
 
@@ -81,12 +89,62 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
   "userfield"		user field set via SetCDRUserField 
 ----------------------------------------------------------*/
 
-static char *desc = "Comma Separated Values CDR Backend";
-
 static char *name = "csv";
 
 static FILE *mf = NULL;
 
+
+static int load_config(void)
+{
+	struct ast_config *cfg;
+	struct ast_variable *var;
+	const char *tmp;
+
+	usegmtime = 0;
+	loguniqueid = 0;
+	loguserfield = 0;
+	
+	cfg = ast_config_load(config);
+	
+	if (!cfg) {
+		ast_log(LOG_WARNING, "unable to load config: %s\n", config);
+		return 0;
+	} 
+	
+	var = ast_variable_browse(cfg, "csv");
+	if (!var) {
+		ast_config_destroy(cfg);
+		return 0;
+	}
+	
+	tmp = ast_variable_retrieve(cfg, "csv", "usegmtime");
+	if (tmp) {
+		usegmtime = ast_true(tmp);
+		if (usegmtime) {
+			ast_log(LOG_DEBUG, "logging time in GMT\n");
+		}
+	}
+
+	tmp = ast_variable_retrieve(cfg, "csv", "loguniqueid");
+	if (tmp) {
+		loguniqueid = ast_true(tmp);
+		if (loguniqueid) {
+			ast_log(LOG_DEBUG, "logging CDR field UNIQUEID\n");
+		}
+	}
+
+	tmp = ast_variable_retrieve(cfg, "csv", "loguserfield");
+	if (tmp) {
+		loguserfield = ast_true(tmp);
+		if (loguserfield) {
+			ast_log(LOG_DEBUG, "logging CDR user-defined field\n");
+		}
+	}
+
+	ast_config_destroy(cfg);
+	return 1;
+}
+
 static int append_string(char *buf, char *s, size_t bufsize)
 {
 	int pos = strlen(buf);
@@ -138,7 +196,11 @@ static int append_date(char *buf, struct timeval tv, size_t bufsize)
 		strncat(buf, ",", bufsize - strlen(buf) - 1);
 		return 0;
 	}
-	localtime_r(&t,&tm);
+	if (usegmtime) {
+		gmtime_r(&t,&tm);
+	} else {
+		ast_localtime(&t, &tm, NULL);
+	}
 	strftime(tmp, sizeof(tmp), DATE_FORMAT, &tm);
 	return append_string(buf, tmp, bufsize);
 }
@@ -179,15 +241,12 @@ static int build_csv_record(char *buf, size_t bufsize, struct ast_cdr *cdr)
 	append_string(buf, ast_cdr_disp2str(cdr->disposition), bufsize);
 	/* AMA Flags */
 	append_string(buf, ast_cdr_flags2str(cdr->amaflags), bufsize);
-
-#ifdef CSV_LOGUNIQUEID
 	/* Unique ID */
-	append_string(buf, cdr->uniqueid, bufsize);
-#endif
-#ifdef CSV_LOGUSERFIELD
+	if (loguniqueid)
+		append_string(buf, cdr->uniqueid, bufsize);
 	/* append the user field */
-	append_string(buf, cdr->userfield,bufsize);	
-#endif
+	if(loguserfield)
+		append_string(buf, cdr->userfield,bufsize);	
 	/* If we hit the end of our buffer, log an error */
 	if (strlen(buf) < bufsize - 5) {
 		/* Trim off trailing comma */
@@ -200,7 +259,7 @@ static int build_csv_record(char *buf, size_t bufsize, struct ast_cdr *cdr)
 
 static int writefile(char *s, char *acc)
 {
-	char tmp[AST_CONFIG_MAX_PATH];
+	char tmp[PATH_MAX];
 	FILE *f;
 	if (strchr(acc, '/') || (acc[0] == '.')) {
 		ast_log(LOG_WARNING, "Account code '%s' insecure for writing file\n", acc);
@@ -221,7 +280,7 @@ static int csv_log(struct ast_cdr *cdr)
 {
 	/* Make sure we have a big enough buf */
 	char buf[1024];
-	char csvmaster[AST_CONFIG_MAX_PATH];
+	char csvmaster[PATH_MAX];
 	snprintf(csvmaster, sizeof(csvmaster),"%s/%s/%s", ast_config_AST_LOG_DIR, CSV_LOG_DIR, CSV_MASTER);
 #if 0
 	printf("[CDR] %s ('%s' -> '%s') Dur: %ds Bill: %ds Disp: %s Flags: %s Account: [%s]\n", cdr->channel, cdr->src, cdr->dst, cdr->duration, cdr->billsec, ast_cdr_disp2str(cdr->disposition), ast_cdr_flags2str(cdr->amaflags), cdr->accountcode);
@@ -250,12 +309,7 @@ static int csv_log(struct ast_cdr *cdr)
 	return 0;
 }
 
-char *description(void)
-{
-	return desc;
-}
-
-int unload_module(void)
+static int unload_module(void)
 {
 	if (mf)
 		fclose(mf);
@@ -263,11 +317,14 @@ int unload_module(void)
 	return 0;
 }
 
-int load_module(void)
+static int load_module(void)
 {
 	int res;
+	
+	if(!load_config())
+		return AST_MODULE_LOAD_DECLINE;
 
-	res = ast_cdr_register(name, desc, csv_log);
+	res = ast_cdr_register(name, ast_module_info->description, csv_log);
 	if (res) {
 		ast_log(LOG_ERROR, "Unable to register CSV CDR handling\n");
 		if (mf)
@@ -276,17 +333,15 @@ int load_module(void)
 	return res;
 }
 
-int reload(void)
+static int reload(void)
 {
+	load_config();
+
 	return 0;
 }
 
-int usecount(void)
-{
-	return 0;
-}
-
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Comma Separated Values CDR Backend",
+		.load = load_module,
+		.unload = unload_module,
+		.reload = reload,
+	       );
