@@ -17,12 +17,18 @@
  */
 
 /*! \file
+ *
+ * \author Mark Spencer <markster@digium.com>
  *
  * \brief Local Proxy Channel
  * 
  * \ingroup channel_drivers
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 79902 $")
+
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -35,10 +41,6 @@
 #include <arpa/inet.h>
 #include <sys/signal.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 73318 $")
-
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
 #include "asterisk/config.h"
@@ -57,76 +59,109 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 73318 $")
 #include "asterisk/app.h"
 #include "asterisk/musiconhold.h"
 #include "asterisk/manager.h"
+#include "asterisk/stringfields.h"
+#include "asterisk/devicestate.h"
 
-static const char desc[] = "Local Proxy Channel";
-static const char type[] = "Local";
 static const char tdesc[] = "Local Proxy Channel Driver";
 
-static int usecnt =0;
-AST_MUTEX_DEFINE_STATIC(usecnt_lock);
-
 #define IS_OUTBOUND(a,b) (a == b->chan ? 1 : 0)
 
-/* Protect the interface list (of sip_pvt's) */
-AST_MUTEX_DEFINE_STATIC(locallock);
-
 static struct ast_channel *local_request(const char *type, int format, void *data, int *cause);
-static int local_digit(struct ast_channel *ast, char digit);
+static int local_digit_begin(struct ast_channel *ast, char digit);
+static int local_digit_end(struct ast_channel *ast, char digit, unsigned int duration);
 static int local_call(struct ast_channel *ast, char *dest, int timeout);
 static int local_hangup(struct ast_channel *ast);
 static int local_answer(struct ast_channel *ast);
 static struct ast_frame *local_read(struct ast_channel *ast);
 static int local_write(struct ast_channel *ast, struct ast_frame *f);
-static int local_indicate(struct ast_channel *ast, int condition);
+static int local_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen);
 static int local_fixup(struct ast_channel *oldchan, struct ast_channel *newchan);
 static int local_sendhtml(struct ast_channel *ast, int subclass, const char *data, int datalen);
+static int local_sendtext(struct ast_channel *ast, const char *text);
+static int local_devicestate(void *data);
 
 /* PBX interface structure for channel registration */
 static const struct ast_channel_tech local_tech = {
-	.type = type,
+	.type = "Local",
 	.description = tdesc,
 	.capabilities = -1,
 	.requester = local_request,
-	.send_digit = local_digit,
+	.send_digit_begin = local_digit_begin,
+	.send_digit_end = local_digit_end,
 	.call = local_call,
 	.hangup = local_hangup,
 	.answer = local_answer,
 	.read = local_read,
 	.write = local_write,
+	.write_video = local_write,
 	.exception = local_read,
 	.indicate = local_indicate,
 	.fixup = local_fixup,
 	.send_html = local_sendhtml,
+	.send_text = local_sendtext,
+	.devicestate = local_devicestate,
 };
 
-static struct local_pvt {
+struct local_pvt {
 	ast_mutex_t lock;			/* Channel private lock */
+	unsigned int flags;                     /* Private flags */
 	char context[AST_MAX_CONTEXT];		/* Context to call */
 	char exten[AST_MAX_EXTENSION];		/* Extension to call */
 	int reqformat;				/* Requested format */
-	int glaredetect;			/* Detect glare on hangup */
-	int cancelqueue;			/* Cancel queue */
-	int alreadymasqed;			/* Already masqueraded */
-	int launchedpbx;			/* Did we launch the PBX */
-	int nooptimization;			/* Don't leave masq state */
 	struct ast_channel *owner;		/* Master Channel */
 	struct ast_channel *chan;		/* Outbound channel */
-	struct local_pvt *next;			/* Next entity */
-} *locals = NULL;
+	struct ast_module_user *u_owner;	/*! reference to keep the module loaded while in use */
+	struct ast_module_user *u_chan;		/*! reference to keep the module loaded while in use */
+	AST_LIST_ENTRY(local_pvt) list;		/* Next entity */
+};
+
+#define LOCAL_GLARE_DETECT    (1 << 0) /*!< Detect glare on hangup */
+#define LOCAL_CANCEL_QUEUE    (1 << 1) /*!< Cancel queue */
+#define LOCAL_ALREADY_MASQED  (1 << 2) /*!< Already masqueraded */
+#define LOCAL_LAUNCHED_PBX    (1 << 3) /*!< PBX was launched */
+#define LOCAL_NO_OPTIMIZATION (1 << 4) /*!< Do not optimize using masquerading */
+
+static AST_LIST_HEAD_STATIC(locals, local_pvt);
+
+/*! \brief Adds devicestate to local channels */
+static int local_devicestate(void *data)
+{
+	char *exten = ast_strdupa(data);
+	char *context = NULL, *opts = NULL;
+	int res;
+
+	if (!(context = strchr(exten, '@'))) {
+		ast_log(LOG_WARNING, "Someone used Local/%s somewhere without a @context. This is bad.\n", exten);
+		return AST_DEVICE_INVALID;	
+	}
+
+	*context++ = '\0';
+
+	/* Strip options if they exist */
+	if ((opts = strchr(context, '/')))
+		*opts = '\0';
+
+	if (option_debug > 2)
+		ast_log(LOG_DEBUG, "Checking if extension %s@%s exists (devicestate)\n", exten, context);
+	res = ast_exists_extension(NULL, context, exten, 1, NULL);
+	if (!res)		
+		return AST_DEVICE_INVALID;
+	else
+		return AST_DEVICE_UNKNOWN;
+}
 
 static int local_queue_frame(struct local_pvt *p, int isoutbound, struct ast_frame *f, struct ast_channel *us)
 {
-	struct ast_channel *other;
+	struct ast_channel *other = NULL;
+
 retrylock:		
+
 	/* Recalculate outbound channel */
-	if (isoutbound) {
-		other = p->owner;
-	} else {
-		other = p->chan;
-	}
+	other = isoutbound ? p->owner : p->chan;
+
 	/* Set glare detection */
-	p->glaredetect = 1;
-	if (p->cancelqueue) {
+	ast_set_flag(p, LOCAL_GLARE_DETECT);
+	if (ast_test_flag(p, LOCAL_CANCEL_QUEUE)) {
 		/* We had a glare on the hangup.  Forget all this business,
 		return and destroy p.  */
 		ast_mutex_unlock(&p->lock);
@@ -135,7 +170,7 @@ retrylock:
 		return -1;
 	}
 	if (!other) {
-		p->glaredetect = 0;
+		ast_clear_flag(p, LOCAL_GLARE_DETECT);
 		return 0;
 	}
 	if (ast_mutex_trylock(&other->lock)) {
@@ -158,7 +193,7 @@ retrylock:
 	}
 	ast_queue_frame(other, f);
 	ast_mutex_unlock(&other->lock);
-	p->glaredetect = 0;
+	ast_clear_flag(p, LOCAL_GLARE_DETECT);
 	return 0;
 }
 
@@ -186,9 +221,7 @@ static int local_answer(struct ast_channel *ast)
 
 static void check_bridge(struct local_pvt *p, int isoutbound)
 {
-	if (p->alreadymasqed || p->nooptimization)
-		return;
-	if (!p->chan || !p->owner || (p->chan->_bridge != ast_bridged_channel(p->chan)))
+	if (ast_test_flag(p, LOCAL_ALREADY_MASQED) || ast_test_flag(p, LOCAL_NO_OPTIMIZATION) || !p->chan || !p->owner || (p->chan->_bridge != ast_bridged_channel(p->chan)))
 		return;
 
 	/* only do the masquerade if we are being called on the outbound channel,
@@ -196,7 +229,7 @@ static void check_bridge(struct local_pvt *p, int isoutbound)
 	   frames on the owner channel (because they would be transferred to the
 	   outbound channel during the masquerade)
 	*/
-	if (isoutbound && p->chan->_bridge /* Not ast_bridged_channel!  Only go one step! */ && !p->owner->readq) {
+	if (isoutbound && p->chan->_bridge /* Not ast_bridged_channel!  Only go one step! */ && AST_LIST_EMPTY(&p->owner->readq)) {
 		/* Masquerade bridged channel into owner */
 		/* Lock everything we need, one by one, and give up if
 		   we can't get everything.  Remember, we'll get another
@@ -206,7 +239,7 @@ static void check_bridge(struct local_pvt *p, int isoutbound)
 				if (!ast_mutex_trylock(&p->owner->lock)) {
 					if (!p->owner->_softhangup) {
 						ast_channel_masquerade(p->owner, p->chan->_bridge);
-						p->alreadymasqed = 1;
+						ast_set_flag(p, LOCAL_ALREADY_MASQED);
 					}
 					ast_mutex_unlock(&p->owner->lock);
 				}
@@ -218,14 +251,14 @@ static void check_bridge(struct local_pvt *p, int isoutbound)
 	   when the local channels go away.
 	*/
 #if 0
-	} else if (!isoutbound && p->owner && p->owner->_bridge && p->chan && !p->chan->readq) {
+	} else if (!isoutbound && p->owner && p->owner->_bridge && p->chan && AST_LIST_EMPTY(&p->chan->readq)) {
 		/* Masquerade bridged channel into chan */
 		if (!ast_mutex_trylock(&(p->owner->_bridge)->lock)) {
 			if (!p->owner->_bridge->_softhangup) {
 				if (!ast_mutex_trylock(&p->chan->lock)) {
 					if (!p->chan->_softhangup) {
 						ast_channel_masquerade(p->chan, p->owner->_bridge);
-						p->alreadymasqed = 1;
+						ast_set_flag(p, LOCAL_ALREADY_MASQED);
 					}
 					ast_mutex_unlock(&p->chan->lock);
 				}
@@ -238,9 +271,7 @@ static void check_bridge(struct local_pvt *p, int isoutbound)
 
 static struct ast_frame  *local_read(struct ast_channel *ast)
 {
-	static struct ast_frame null = { AST_FRAME_NULL, };
-
-	return &null;
+	return &ast_null_frame;
 }
 
 static int local_write(struct ast_channel *ast, struct ast_frame *f)
@@ -255,12 +286,13 @@ static int local_write(struct ast_channel *ast, struct ast_frame *f)
 	/* Just queue for delivery to the other side */
 	ast_mutex_lock(&p->lock);
 	isoutbound = IS_OUTBOUND(ast, p);
-	if (f && (f->frametype == AST_FRAME_VOICE)) 
+	if (f && (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_VIDEO))
 		check_bridge(p, isoutbound);
-	if (!p->alreadymasqed)
+	if (!ast_test_flag(p, LOCAL_ALREADY_MASQED))
 		res = local_queue_frame(p, isoutbound, f, ast);
 	else {
-		ast_log(LOG_DEBUG, "Not posting to queue since already masked on '%s'\n", ast->name);
+		if (option_debug)
+			ast_log(LOG_DEBUG, "Not posting to queue since already masked on '%s'\n", ast->name);
 		res = 0;
 	}
 	if (!res)
@@ -290,30 +322,40 @@ static int local_fixup(struct ast_channel *oldchan, struct ast_channel *newchan)
 	return 0;
 }
 
-static int local_indicate(struct ast_channel *ast, int condition)
+static int local_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen)
 {
 	struct local_pvt *p = ast->tech_pvt;
-	int res = -1;
+	int res = 0;
 	struct ast_frame f = { AST_FRAME_CONTROL, };
 	int isoutbound;
 
 	if (!p)
 		return -1;
 
-	/* Queue up a frame representing the indication as a control frame */
-	ast_mutex_lock(&p->lock);
-	isoutbound = IS_OUTBOUND(ast, p);
-	f.subclass = condition;
-	if (!(res = local_queue_frame(p, isoutbound, &f, ast)))
-		ast_mutex_unlock(&p->lock);
+	/* If this is an MOH hold or unhold, do it on the Local channel versus real channel */
+	if (condition == AST_CONTROL_HOLD) {
+		ast_moh_start(ast, data, NULL);
+	} else if (condition == AST_CONTROL_UNHOLD) {
+		ast_moh_stop(ast);
+	} else {
+		/* Queue up a frame representing the indication as a control frame */
+		ast_mutex_lock(&p->lock);
+		isoutbound = IS_OUTBOUND(ast, p);
+		f.subclass = condition;
+		f.data = (void*)data;
+		f.datalen = datalen;
+		if (!(res = local_queue_frame(p, isoutbound, &f, ast)))
+			ast_mutex_unlock(&p->lock);
+	}
+
 	return res;
 }
 
-static int local_digit(struct ast_channel *ast, char digit)
+static int local_digit_begin(struct ast_channel *ast, char digit)
 {
 	struct local_pvt *p = ast->tech_pvt;
 	int res = -1;
-	struct ast_frame f = { AST_FRAME_DTMF, };
+	struct ast_frame f = { AST_FRAME_DTMF_BEGIN, };
 	int isoutbound;
 
 	if (!p)
@@ -322,6 +364,46 @@ static int local_digit(struct ast_channel *ast, char digit)
 	ast_mutex_lock(&p->lock);
 	isoutbound = IS_OUTBOUND(ast, p);
 	f.subclass = digit;
+	if (!(res = local_queue_frame(p, isoutbound, &f, ast)))
+		ast_mutex_unlock(&p->lock);
+
+	return res;
+}
+
+static int local_digit_end(struct ast_channel *ast, char digit, unsigned int duration)
+{
+	struct local_pvt *p = ast->tech_pvt;
+	int res = -1;
+	struct ast_frame f = { AST_FRAME_DTMF_END, };
+	int isoutbound;
+
+	if (!p)
+		return -1;
+
+	ast_mutex_lock(&p->lock);
+	isoutbound = IS_OUTBOUND(ast, p);
+	f.subclass = digit;
+	f.len = duration;
+	if (!(res = local_queue_frame(p, isoutbound, &f, ast)))
+		ast_mutex_unlock(&p->lock);
+
+	return res;
+}
+
+static int local_sendtext(struct ast_channel *ast, const char *text)
+{
+	struct local_pvt *p = ast->tech_pvt;
+	int res = -1;
+	struct ast_frame f = { AST_FRAME_TEXT, };
+	int isoutbound;
+
+	if (!p)
+		return -1;
+
+	ast_mutex_lock(&p->lock);
+	isoutbound = IS_OUTBOUND(ast, p);
+	f.data = (char *) text;
+	f.datalen = strlen(text) + 1;
 	if (!(res = local_queue_frame(p, isoutbound, &f, ast)))
 		ast_mutex_unlock(&p->lock);
 	return res;
@@ -347,8 +429,8 @@ static int local_sendhtml(struct ast_channel *ast, int subclass, const char *dat
 	return res;
 }
 
-/*--- local_call: Initiate new call, part of PBX interface */
-/* 	dest is the dial string */
+/*! \brief Initiate new call, part of PBX interface 
+ * 	dest is the dial string */
 static int local_call(struct ast_channel *ast, char *dest, int timeout)
 {
 	struct local_pvt *p = ast->tech_pvt;
@@ -361,19 +443,17 @@ static int local_call(struct ast_channel *ast, char *dest, int timeout)
 	
 	ast_mutex_lock(&p->lock);
 
-	ast_set_callerid(p->chan,
-		p->owner->cid.cid_num, p->owner->cid.cid_name,
-		p->owner->cid.cid_ani);
-
-	if (p->owner->cid.cid_rdnis)
-		p->chan->cid.cid_rdnis = strdup(p->owner->cid.cid_rdnis);
-	else
-		p->chan->cid.cid_rdnis = NULL;
-
+	/*
+	 * Note that cid_num and cid_name aren't passed in the ast_channel_alloc
+	 * call, so it's done here instead.
+	 */
+	p->chan->cid.cid_num = ast_strdup(p->owner->cid.cid_num);
+	p->chan->cid.cid_name = ast_strdup(p->owner->cid.cid_name);
+	p->chan->cid.cid_rdnis = ast_strdup(p->owner->cid.cid_rdnis);
+	p->chan->cid.cid_ani = ast_strdup(p->owner->cid.cid_ani);
 	p->chan->cid.cid_pres = p->owner->cid.cid_pres;
-
-	strncpy(p->chan->language, p->owner->language, sizeof(p->chan->language) - 1);
-	strncpy(p->chan->accountcode, p->owner->accountcode, sizeof(p->chan->accountcode) - 1);
+	ast_string_field_set(p->chan, language, p->owner->language);
+	ast_string_field_set(p->chan, accountcode, p->owner->accountcode);
 	p->chan->cdrflags = p->owner->cdrflags;
 
 	/* copy the channel variables from the incoming channel to the outgoing channel */
@@ -381,58 +461,29 @@ static int local_call(struct ast_channel *ast, char *dest, int timeout)
 	AST_LIST_TRAVERSE(&p->owner->varshead, varptr, entries) {
 		namelen = strlen(varptr->name);
 		len = sizeof(struct ast_var_t) + namelen + strlen(varptr->value) + 2;
-		new = malloc(len);
-		if (new) {
+		if ((new = ast_calloc(1, len))) {
 			memcpy(new, varptr, len);
 			new->value = &(new->name[0]) + namelen + 1;
 			AST_LIST_INSERT_TAIL(&p->chan->varshead, new, entries);
-		} else {
-			ast_log(LOG_ERROR, "Out of memory!\n");
 		}
 	}
 
 	/* Start switch on sub channel */
 	if (!(res = ast_pbx_start(p->chan)))
-		p->launchedpbx = 1;
+		ast_set_flag(p, LOCAL_LAUNCHED_PBX);
 
 	ast_mutex_unlock(&p->lock);
 	return res;
 }
 
-#if 0
-static void local_destroy(struct local_pvt *p)
-{
-	struct local_pvt *cur, *prev = NULL;
-	ast_mutex_lock(&locallock);
-	cur = locals;
-	while(cur) {
-		if (cur == p) {
-			if (prev)
-				prev->next = cur->next;
-			else
-				locals = cur->next;
-			ast_mutex_destroy(cur);
-			free(cur);
-			break;
-		}
-		prev = cur;
-		cur = cur->next;
-	}
-	ast_mutex_unlock(&locallock);
-	if (!cur)
-		ast_log(LOG_WARNING, "Unable ot find local '%s@%s' in local list\n", p->exten, p->context);
-}
-#endif
-
-/*--- local_hangup: Hangup a call through the local proxy channel */
+/*! \brief Hangup a call through the local proxy channel */
 static int local_hangup(struct ast_channel *ast)
 {
 	struct local_pvt *p = ast->tech_pvt;
 	int isoutbound;
 	struct ast_frame f = { AST_FRAME_CONTROL, AST_CONTROL_HANGUP };
-	struct local_pvt *cur, *prev=NULL;
 	struct ast_channel *ochan = NULL;
-	int glaredetect, res = 0;
+	int glaredetect = 0, res = 0;
 
 	if (!p)
 		return -1;
@@ -440,39 +491,31 @@ static int local_hangup(struct ast_channel *ast)
 	ast_mutex_lock(&p->lock);
 	isoutbound = IS_OUTBOUND(ast, p);
 	if (isoutbound) {
+		const char *status = pbx_builtin_getvar_helper(p->chan, "DIALSTATUS");
+		if ((status) && (p->owner))
+			pbx_builtin_setvar_helper(p->owner, "CHANLOCALSTATUS", status);
 		p->chan = NULL;
-		p->launchedpbx = 0;
-	} else
+		ast_clear_flag(p, LOCAL_LAUNCHED_PBX);
+		ast_module_user_remove(p->u_chan);
+	} else {
 		p->owner = NULL;
-	ast->tech_pvt = NULL;
+		ast_module_user_remove(p->u_owner);
+	}
 	
-	ast_mutex_lock(&usecnt_lock);
-	usecnt--;
-	ast_mutex_unlock(&usecnt_lock);
+	ast->tech_pvt = NULL;
 	
 	if (!p->owner && !p->chan) {
 		/* Okay, done with the private part now, too. */
-		glaredetect = p->glaredetect;
+		glaredetect = ast_test_flag(p, LOCAL_GLARE_DETECT);
 		/* If we have a queue holding, don't actually destroy p yet, but
 		   let local_queue do it. */
-		if (p->glaredetect)
-			p->cancelqueue = 1;
+		if (glaredetect)
+			ast_set_flag(p, LOCAL_CANCEL_QUEUE);
 		ast_mutex_unlock(&p->lock);
 		/* Remove from list */
-		ast_mutex_lock(&locallock);
-		cur = locals;
-		while(cur) {
-			if (cur == p) {
-				if (prev)
-					prev->next = cur->next;
-				else
-					locals = cur->next;
-				break;
-			}
-			prev = cur;
-			cur = cur->next;
-		}
-		ast_mutex_unlock(&locallock);
+		AST_LIST_LOCK(&locals);
+		AST_LIST_REMOVE(&locals, p, list);
+		AST_LIST_UNLOCK(&locals);
 		/* Grab / release lock just in case */
 		ast_mutex_lock(&p->lock);
 		ast_mutex_unlock(&p->lock);
@@ -483,7 +526,7 @@ static int local_hangup(struct ast_channel *ast)
 		}
 		return 0;
 	}
-	if (p->chan && !p->launchedpbx)
+	if (p->chan && !ast_test_flag(p, LOCAL_LAUNCHED_PBX))
 		/* Need to actually hangup since there is no PBX */
 		ochan = p->chan;
 	else
@@ -495,59 +538,70 @@ static int local_hangup(struct ast_channel *ast)
 	return 0;
 }
 
-/*--- local_alloc: Create a call structure */
-static struct local_pvt *local_alloc(char *data, int format)
+/*! \brief Create a call structure */
+static struct local_pvt *local_alloc(const char *data, int format)
 {
-	struct local_pvt *tmp;
-	char *c;
-	char *opts;
+	struct local_pvt *tmp = NULL;
+	char *c = NULL, *opts = NULL;
 
-	tmp = malloc(sizeof(struct local_pvt));
-	if (tmp) {
-		memset(tmp, 0, sizeof(struct local_pvt));
-		ast_mutex_init(&tmp->lock);
-		strncpy(tmp->exten, data, sizeof(tmp->exten) - 1);
-		opts = strchr(tmp->exten, '/');
-		if (opts) {
-			*opts='\0';
-			opts++;
-			if (strchr(opts, 'n'))
-				tmp->nooptimization = 1;
-		}
-		c = strchr(tmp->exten, '@');
-		if (c) {
-			*c = '\0';
-			c++;
-			strncpy(tmp->context, c, sizeof(tmp->context) - 1);
-		} else
-			strncpy(tmp->context, "default", sizeof(tmp->context) - 1);
-		tmp->reqformat = format;
-		if (!ast_exists_extension(NULL, tmp->context, tmp->exten, 1, NULL)) {
-			ast_log(LOG_NOTICE, "No such extension/context %s@%s creating local channel\n", tmp->exten, tmp->context);
-			ast_mutex_destroy(&tmp->lock);
-			free(tmp);
-			tmp = NULL;
-		} else {
-			/* Add to list */
-			ast_mutex_lock(&locallock);
-			tmp->next = locals;
-			locals = tmp;
-			ast_mutex_unlock(&locallock);
-		}
-		
+	if (!(tmp = ast_calloc(1, sizeof(*tmp))))
+		return NULL;
+
+	/* Initialize private structure information */
+	ast_mutex_init(&tmp->lock);
+	ast_copy_string(tmp->exten, data, sizeof(tmp->exten));
+
+	/* Look for options */
+	if ((opts = strchr(tmp->exten, '/'))) {
+		*opts++ = '\0';
+		if (strchr(opts, 'n'))
+			ast_set_flag(tmp, LOCAL_NO_OPTIMIZATION);
 	}
+
+	/* Look for a context */
+	if ((c = strchr(tmp->exten, '@')))
+		*c++ = '\0';
+
+	ast_copy_string(tmp->context, c ? c : "default", sizeof(tmp->context));
+
+	tmp->reqformat = format;
+
+	if (!ast_exists_extension(NULL, tmp->context, tmp->exten, 1, NULL)) {
+		ast_log(LOG_NOTICE, "No such extension/context %s@%s creating local channel\n", tmp->exten, tmp->context);
+		ast_mutex_destroy(&tmp->lock);
+		free(tmp);
+		tmp = NULL;
+	} else {
+		/* Add to list */
+		AST_LIST_LOCK(&locals);
+		AST_LIST_INSERT_HEAD(&locals, tmp, list);
+		AST_LIST_UNLOCK(&locals);
+	}
+	
 	return tmp;
 }
 
-/*--- local_new: Start new local channel */
+/*! \brief Start new local channel */
 static struct ast_channel *local_new(struct local_pvt *p, int state)
 {
-	struct ast_channel *tmp, *tmp2;
-	int randnum = rand() & 0xffff, fmt = 0;
+	struct ast_channel *tmp = NULL, *tmp2 = NULL;
+	int randnum = ast_random() & 0xffff, fmt = 0;
+	const char *t;
+	int ama;
 
-	tmp = ast_channel_alloc(1);
-	tmp2 = ast_channel_alloc(1);
-	if (!tmp || !tmp2) {
+	/* Allocate two new Asterisk channels */
+	/* safe accountcode */
+	if (p->owner && p->owner->accountcode)
+		t = p->owner->accountcode;
+	else
+		t = "";
+
+	if (p->owner)
+		ama = p->owner->amaflags;
+	else
+		ama = 0;
+	if (!(tmp = ast_channel_alloc(1, state, 0, 0, t, p->exten, p->context, ama, "Local/%s@%s-%04x,1", p->exten, p->context, randnum)) 
+			|| !(tmp2 = ast_channel_alloc(1, AST_STATE_RING, 0, 0, t, p->exten, p->context, ama, "Local/%s@%s-%04x,2", p->exten, p->context, randnum))) {
 		if (tmp)
 			ast_channel_free(tmp);
 		if (tmp2)
@@ -557,14 +611,11 @@ static struct ast_channel *local_new(struct local_pvt *p, int state)
 	} 
 
 	tmp2->tech = tmp->tech = &local_tech;
+
 	tmp->nativeformats = p->reqformat;
 	tmp2->nativeformats = p->reqformat;
-	snprintf(tmp->name, sizeof(tmp->name), "Local/%s@%s-%04x,1", p->exten, p->context, randnum);
-	snprintf(tmp2->name, sizeof(tmp2->name), "Local/%s@%s-%04x,2", p->exten, p->context, randnum);
-	tmp->type = type;
-	tmp2->type = type;
-	ast_setstate(tmp, state);
-	ast_setstate(tmp2, AST_STATE_RING);
+
+	/* Determine our read/write format and set it on each channel */
 	fmt = ast_best_codec(p->reqformat);
 	tmp->writeformat = fmt;
 	tmp2->writeformat = fmt;
@@ -574,15 +625,15 @@ static struct ast_channel *local_new(struct local_pvt *p, int state)
 	tmp2->readformat = fmt;
 	tmp->rawreadformat = fmt;
 	tmp2->rawreadformat = fmt;
+
 	tmp->tech_pvt = p;
 	tmp2->tech_pvt = p;
+
 	p->owner = tmp;
 	p->chan = tmp2;
-	ast_mutex_lock(&usecnt_lock);
-	usecnt++;
-	usecnt++;
-	ast_mutex_unlock(&usecnt_lock);
-	ast_update_use_count();
+	p->u_owner = ast_module_user_add(p->owner);
+	p->u_chan = ast_module_user_add(p->chan);
+
 	ast_copy_string(tmp->context, p->context, sizeof(tmp->context));
 	ast_copy_string(tmp2->context, p->context, sizeof(tmp2->context));
 	ast_copy_string(tmp2->exten, p->exten, sizeof(tmp->exten));
@@ -593,36 +644,38 @@ static struct ast_channel *local_new(struct local_pvt *p, int state)
 }
 
 
-/*--- local_request: Part of PBX interface */
+/*! \brief Part of PBX interface */
 static struct ast_channel *local_request(const char *type, int format, void *data, int *cause)
 {
-	struct local_pvt *p;
+	struct local_pvt *p = NULL;
 	struct ast_channel *chan = NULL;
 
-	p = local_alloc(data, format);
-	if (p)
+	/* Allocate a new private structure and then Asterisk channel */
+	if ((p = local_alloc(data, format)))
 		chan = local_new(p, AST_STATE_DOWN);
+
 	return chan;
 }
 
-/*--- locals_show: CLI command "local show channels" */
+/*! \brief CLI command "local show channels" */
 static int locals_show(int fd, int argc, char **argv)
 {
-	struct local_pvt *p;
+	struct local_pvt *p = NULL;
 
 	if (argc != 3)
 		return RESULT_SHOWUSAGE;
-	ast_mutex_lock(&locallock);
-	p = locals;
-	while(p) {
-		ast_mutex_lock(&p->lock);
-		ast_cli(fd, "%s -- %s@%s\n", p->owner ? p->owner->name : "<unowned>", p->exten, p->context);
-		ast_mutex_unlock(&p->lock);
-		p = p->next;
-	}
-	if (!locals)
+
+	AST_LIST_LOCK(&locals);
+	if (!AST_LIST_EMPTY(&locals)) {
+		AST_LIST_TRAVERSE(&locals, p, list) {
+			ast_mutex_lock(&p->lock);
+			ast_cli(fd, "%s -- %s@%s\n", p->owner ? p->owner->name : "<unowned>", p->exten, p->context);
+			ast_mutex_unlock(&p->lock);
+		}
+	} else
 		ast_cli(fd, "No local channels in use\n");
-	ast_mutex_unlock(&locallock);
+	AST_LIST_UNLOCK(&locals);
+
 	return RESULT_SUCCESS;
 }
 
@@ -630,46 +683,40 @@ static char show_locals_usage[] =
 "Usage: local show channels\n"
 "       Provides summary information on active local proxy channels.\n";
 
-static struct ast_cli_entry cli_show_locals = {
-	{ "local", "show", "channels", NULL }, locals_show, 
-	"Show status of local channels", show_locals_usage, NULL };
+static struct ast_cli_entry cli_local[] = {
+	{ { "local", "show", "channels", NULL },
+	locals_show, "List status of local channels",
+	show_locals_usage },
+};
 
-/*--- load_module: Load module into PBX, register channel */
-int load_module()
+/*! \brief Load module into PBX, register channel */
+static int load_module(void)
 {
 	/* Make sure we can register our channel type */
 	if (ast_channel_register(&local_tech)) {
-		ast_log(LOG_ERROR, "Unable to register channel class %s\n", type);
+		ast_log(LOG_ERROR, "Unable to register channel class 'Local'\n");
 		return -1;
 	}
-	ast_cli_register(&cli_show_locals);
+	ast_cli_register_multiple(cli_local, sizeof(cli_local) / sizeof(struct ast_cli_entry));
 	return 0;
 }
 
-/*--- reload: Reload module */
-int reload()
+/*! \brief Unload the local proxy channel from Asterisk */
+static int unload_module(void)
 {
-	return 0;
-}
-
-/*--- unload_module: Unload the local proxy channel from Asterisk */
-int unload_module()
-{
-	struct local_pvt *p;
+	struct local_pvt *p = NULL;
 
 	/* First, take us out of the channel loop */
-	ast_cli_unregister(&cli_show_locals);
+	ast_cli_unregister_multiple(cli_local, sizeof(cli_local) / sizeof(struct ast_cli_entry));
 	ast_channel_unregister(&local_tech);
-	if (!ast_mutex_lock(&locallock)) {
+	if (!AST_LIST_LOCK(&locals)) {
 		/* Hangup all interfaces if they have an owner */
-		p = locals;
-		while(p) {
+		AST_LIST_TRAVERSE(&locals, p, list) {
 			if (p->owner)
 				ast_softhangup(p->owner, AST_SOFTHANGUP_APPUNLOAD);
-			p = p->next;
 		}
-		locals = NULL;
-		ast_mutex_unlock(&locallock);
+		AST_LIST_UNLOCK(&locals);
+		AST_LIST_HEAD_DESTROY(&locals);
 	} else {
 		ast_log(LOG_WARNING, "Unable to lock the monitor\n");
 		return -1;
@@ -677,18 +724,4 @@ int unload_module()
 	return 0;
 }
 
-int usecount()
-{
-	return usecnt;
-}
-
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
-
-char *description()
-{
-	return (char *) desc;
-}
-
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Local Proxy Channel");
