@@ -53,18 +53,33 @@ enum misdn_cfg_elements {
 	MISDN_CFG_INCOMING_EARLY_AUDIO,      /* int (bool) */
 	MISDN_CFG_ECHOCANCEL,          /* int */
 #ifdef MISDN_1_2
-	MISDN_CFG_PIPELINE,            /* char[] */
+	MISDN_CFG_PIPELINE,      /* char[] */
+#endif
+
+#ifdef WITH_BEROEC
+	MISDN_CFG_BNECHOCANCEL,
+	MISDN_CFG_BNEC_ANTIHOWL,
+	MISDN_CFG_BNEC_NLP,
+	MISDN_CFG_BNEC_ZEROCOEFF,
+	MISDN_CFG_BNEC_TD,
+	MISDN_CFG_BNEC_ADAPT,
 #endif
 	MISDN_CFG_NEED_MORE_INFOS,     /* bool */
 	MISDN_CFG_NOAUTORESPOND_ON_SETUP,     /* bool */
-	MISDN_CFG_OVERLAP_DIAL,        /* int (bool) */
 	MISDN_CFG_NTTIMEOUT,     	/* bool */
 	MISDN_CFG_BRIDGING,              /* bool */
 	MISDN_CFG_JITTERBUFFER,             /* int */
 	MISDN_CFG_JITTERBUFFER_UPPER_THRESHOLD,              /* int */
 	MISDN_CFG_CALLGROUP,           /* ast_group_t */
 	MISDN_CFG_PICKUPGROUP,         /* ast_group_t */
+	MISDN_CFG_MAX_IN,              /* int */
+	MISDN_CFG_MAX_OUT,              /* int */
+	MISDN_CFG_L1_TIMEOUT,          /* int */
+	MISDN_CFG_OVERLAP_DIAL, 	/* int (bool)*/
 	MISDN_CFG_MSNS,                /* char[] */
+	MISDN_CFG_FAXDETECT,           /* char[] */
+ 	MISDN_CFG_FAXDETECT_CONTEXT,   /* char[] */
+ 	MISDN_CFG_FAXDETECT_TIMEOUT,   /* int */
 	MISDN_CFG_PTP,                 /* int (bool) */
 	MISDN_CFG_LAST,
 	
@@ -104,6 +119,15 @@ void misdn_cfg_update_ptp( void );
  * case of a char* only its first byte will be nulled). */
 void misdn_cfg_get(int port, enum misdn_cfg_elements elem, void* buf, int bufsize);
 
+/* returns the enum element for the given name, returns MISDN_CFG_FIRST if none was found */
+enum misdn_cfg_elements misdn_cfg_get_elem (char *name);
+
+/* fills the buffer with the name of the given config element */
+void misdn_cfg_get_name (enum misdn_cfg_elements elem, void *buf, int bufsize);
+
+/* fills the buffer with the description of the given config element */
+void misdn_cfg_get_desc (enum misdn_cfg_elements elem, void *buf, int bufsize, void *buf_default, int bufsize_default);
+
 /* fills the buffer with a ',' separated list of all active ports */
 void misdn_cfg_get_ports_string(char *ports);
 
@@ -118,4 +142,9 @@ int misdn_cfg_is_msn_valid(int port, char* msn);
 int misdn_cfg_is_port_valid(int port);
 int misdn_cfg_is_group_method(char *group, enum misdn_cfg_method meth);
 
+#if 0
+char *misdn_cfg_get_next_group(char *group);
+int misdn_cfg_get_next_port_in_group(int port, char *group);
+#endif
+
 #endif
