@@ -14,6 +14,7 @@
 #ifndef TE_LIB
 #define TE_LIB
 
+#include <mISDNuser/suppserv.h>
 
 /** For initialization usage **/
 /* typedef int ie_nothing_t ;*/
@@ -149,6 +150,7 @@ enum event_e {
 	EVENT_NEW_L3ID,
 	EVENT_NEW_BC,
 	EVENT_PORT_ALARM,
+	EVENT_NEW_CHANNEL,
 	EVENT_UNKNOWN
 }; 
 
@@ -196,22 +198,6 @@ enum layer_e {
 
 
 
-/** FACILITY STUFF **/
-
-enum facility_type {
-	FACILITY_NONE,
-	FACILITY_CALLDEFLECT=0x91,
-	FACILITY_CENTREX=0x88
-};
-
-union facility {
-	char calldeflect_nr[15];
-	char cnip[256];
-};
-
-
-
-
 struct misdn_bchannel {
 	struct send_lock *send_lock;
 
@@ -247,7 +233,7 @@ struct misdn_bchannel {
 	int cw;
 	int addr;
 
-	unsigned char * bframe;
+	char * bframe;
 	int bframe_len;
 	int time_usec;
 	
@@ -284,12 +270,16 @@ struct misdn_bchannel {
 	int progress_coding;
 	int progress_location;
 	int progress_indicator;
-	
-	enum facility_type fac_type;
-	union facility fac;
-	
-	enum facility_type out_fac_type;
-	union facility out_fac;
+
+	struct FacParm fac_in;
+	struct FacParm fac_out;
+
+	/* storing the current AOCD info here */
+	enum FacFunction AOCDtype;
+	union {
+		struct FacAOCDCurrency currency;
+		struct FacAOCDChargingUnit chargingUnit;
+	} AOCD;
 	
 	enum event_e evq;
 	
@@ -350,7 +340,9 @@ struct misdn_bchannel {
 /* 	unsigned char info_keypad[32]; */
 /* 	unsigned char clisub[24]; */
 /* 	unsigned char cldsub[24]; */
-/* 	unsigned char uu[256]; */
+
+ 	char uu[256];
+	int uulen;
   
 	int cause;
 	int out_cause;
@@ -364,7 +356,6 @@ struct misdn_bchannel {
 #else
 	int ec_enable;
 	int ec_deftaps;
-	int ec_training;
 #endif
 	
 	int channel_found;
@@ -383,7 +374,6 @@ void (*cb_log) (int level, int port, char *tmpl, ...);
 int (*cb_jb_empty)(struct misdn_bchannel *bc, char *buffer, int len);
 
 struct misdn_lib_iface {
-	
 	enum event_response_e (*cb_event)(enum event_e event, struct misdn_bchannel *bc, void *user_data);
 	void (*cb_log)(int level, int port, char *tmpl, ...);
 	int (*cb_jb_empty)(struct misdn_bchannel *bc, char *buffer, int len);
@@ -397,6 +387,8 @@ int misdn_lib_init(char *portlist, struct misdn_lib_iface* iface, void *user_dat
 int misdn_lib_send_event(struct misdn_bchannel *bc, enum event_e event );
 void misdn_lib_destroy(void);
 
+void misdn_lib_isdn_l1watcher(int port);
+
 void misdn_lib_log_ies(struct misdn_bchannel *bc);
 
 char *manager_isdn_get_info(enum event_e event);
@@ -412,6 +404,10 @@ int misdn_lib_tx2misdn_frm(struct misdn_bchannel *bc, void *data, int len);
 
 void manager_ph_control(struct misdn_bchannel *bc, int c1, int c2);
 
+void isdn_lib_update_rxgain (struct misdn_bchannel *bc);
+void isdn_lib_update_txgain (struct misdn_bchannel *bc);
+void isdn_lib_update_ec (struct misdn_bchannel *bc);
+void isdn_lib_stop_dtmf (struct misdn_bchannel *bc);
 
 int misdn_lib_port_restart(int port);
 int misdn_lib_pid_restart(int pid);
@@ -423,6 +419,8 @@ int misdn_lib_is_port_blocked(int port);
 int misdn_lib_port_block(int port);
 int misdn_lib_port_unblock(int port);
 
+int misdn_lib_port_is_pri(int port);
+
 int misdn_lib_port_up(int port, int notcheck);
 
 int misdn_lib_get_port_down(int port);
@@ -436,9 +434,6 @@ void misdn_lib_release(struct misdn_bchannel *bc);
 int misdn_cap_is_speech(int cap);
 int misdn_inband_avail(struct misdn_bchannel *bc);
 
-int misdn_lib_send_facility(struct misdn_bchannel *bc, enum facility_type fac, void *data);
-
-
 void manager_ec_enable(struct misdn_bchannel *bc);
 void manager_ec_disable(struct misdn_bchannel *bc);
 
@@ -476,5 +471,4 @@ void bc_state_change(struct misdn_bchannel *bc, enum bchannel_state state);
 
 void misdn_dump_chanlist(void);
 
-
 #endif
