@@ -26,6 +26,10 @@
  * \ingroup channel_drivers
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 67210 $")
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -33,13 +37,13 @@
 
 #include "chan_misdn_config.h"
 
-#include <asterisk/config.h>
-#include <asterisk/channel.h>
-#include <asterisk/logger.h>
-#include <asterisk/lock.h>
-#include <asterisk/pbx.h>
-#include <asterisk/strings.h>
-#include <asterisk/utils.h>
+#include "asterisk/config.h"
+#include "asterisk/channel.h"
+#include "asterisk/logger.h"
+#include "asterisk/lock.h"
+#include "asterisk/pbx.h"
+#include "asterisk/strings.h"
+#include "asterisk/utils.h"
 
 #define AST_LOAD_CFG ast_config_load
 #define AST_DESTROY_CFG ast_config_destroy
@@ -80,70 +84,283 @@ struct misdn_cfg_spec {
 	enum misdn_cfg_type type;
 	char def[BUFFERSIZE];
 	int boolint_def;
+	char desc[BUFFERSIZE];
 };
 
+
+static const char ports_description[] =
+	"Define your ports, e.g. 1,2 (depends on mISDN-driver loading order).";
+
 static const struct misdn_cfg_spec port_spec[] = {
-	{ "name", MISDN_CFG_GROUPNAME, MISDN_CTYPE_STR, "default", NONE },
-	{ "allowed_bearers", MISDN_CFG_ALLOWED_BEARERS, MISDN_CTYPE_STR, "all", NONE },
-	{ "rxgain", MISDN_CFG_RXGAIN, MISDN_CTYPE_INT, "0", NONE },
-	{ "txgain", MISDN_CFG_TXGAIN, MISDN_CTYPE_INT, "0", NONE },
-	{ "te_choose_channel", MISDN_CFG_TE_CHOOSE_CHANNEL, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "far_alerting", MISDN_CFG_FAR_ALERTING, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "pmp_l1_check", MISDN_CFG_PMP_L1_CHECK, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "reject_cause", MISDN_CFG_REJECT_CAUSE, MISDN_CTYPE_INT, "21", NONE },
-	{ "block_on_alarm", MISDN_CFG_ALARM_BLOCK, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "hdlc", MISDN_CFG_HDLC, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "context", MISDN_CFG_CONTEXT, MISDN_CTYPE_STR, "default", NONE },
-	{ "language", MISDN_CFG_LANGUAGE, MISDN_CTYPE_STR, "en", NONE },
-	{ "musicclass", MISDN_CFG_MUSICCLASS, MISDN_CTYPE_STR, "default", NONE },
-	{ "callerid", MISDN_CFG_CALLERID, MISDN_CTYPE_STR, "", NONE },
-	{ "method", MISDN_CFG_METHOD, MISDN_CTYPE_STR, "standard", NONE },
-	{ "dialplan", MISDN_CFG_DIALPLAN, MISDN_CTYPE_INT, "0", NONE },
-	{ "localdialplan", MISDN_CFG_LOCALDIALPLAN, MISDN_CTYPE_INT, "0", NONE },
-	{ "cpndialplan", MISDN_CFG_CPNDIALPLAN, MISDN_CTYPE_INT, "0", NONE },
-	{ "nationalprefix", MISDN_CFG_NATPREFIX, MISDN_CTYPE_STR, "0", NONE },
-	{ "internationalprefix", MISDN_CFG_INTERNATPREFIX, MISDN_CTYPE_STR, "00", NONE },
-	{ "presentation", MISDN_CFG_PRES, MISDN_CTYPE_INT, "-1", NONE },
-	{ "screen", MISDN_CFG_SCREEN, MISDN_CTYPE_INT, "-1", NONE },
-	{ "always_immediate", MISDN_CFG_ALWAYS_IMMEDIATE, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "nodialtone", MISDN_CFG_NODIALTONE, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "immediate", MISDN_CFG_IMMEDIATE, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "senddtmf", MISDN_CFG_SENDDTMF, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "hold_allowed", MISDN_CFG_HOLD_ALLOWED, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "early_bconnect", MISDN_CFG_EARLY_BCONNECT, MISDN_CTYPE_BOOL, "yes", NONE },
-	{ "incoming_early_audio", MISDN_CFG_INCOMING_EARLY_AUDIO, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "echocancel", MISDN_CFG_ECHOCANCEL, MISDN_CTYPE_BOOLINT, "0", 128 },
+	{ "name", MISDN_CFG_GROUPNAME, MISDN_CTYPE_STR, "default", NONE,
+		"Name of the portgroup." },
+	{ "allowed_bearers", MISDN_CFG_ALLOWED_BEARERS, MISDN_CTYPE_STR, "all", NONE,
+		"Here you can define which bearers should be allowed." },
+	{ "rxgain", MISDN_CFG_RXGAIN, MISDN_CTYPE_INT, "0", NONE,
+		"Set this between -8 and 8 to change the RX Gain." },
+	{ "txgain", MISDN_CFG_TXGAIN, MISDN_CTYPE_INT, "0", NONE,
+		"Set this between -8 and 8 to change the TX Gain." },
+	{ "te_choose_channel", MISDN_CFG_TE_CHOOSE_CHANNEL, MISDN_CTYPE_BOOL, "no", NONE,
+		"Some telcos espacially in NL seem to need this set to yes,\n"
+		"\talso in switzerland this seems to be important." },
+	{ "far_alerting", MISDN_CFG_FAR_ALERTING, MISDN_CTYPE_BOOL, "no", NONE,
+		"If we should generate ringing for chan_sip and others." },
+	{ "pmp_l1_check", MISDN_CFG_PMP_L1_CHECK, MISDN_CTYPE_BOOL, "no", NONE,
+		"This option defines, if chan_misdn should check the L1 on a PMP\n"
+		"\tbefore makeing a group call on it. The L1 may go down for PMP Ports\n"
+		"\tso we might need this.\n"
+		"\tBut be aware! a broken or plugged off cable might be used for a group call\n"
+		"\tas well, since chan_misdn has no chance to distinguish if the L1 is down\n"
+		"\tbecause of a lost Link or because the Provider shut it down..." },
+	{ "block_on_alarm", MISDN_CFG_ALARM_BLOCK, MISDN_CTYPE_BOOL, "no", NONE ,
+	  "Block this port if we have an alarm on it."
+	  "default: yes\n" },
+	{ "hdlc", MISDN_CFG_HDLC, MISDN_CTYPE_BOOL, "no", NONE,
+		"Set this to yes, if you want to bridge a mISDN data channel to\n"
+		"\tanother channel type or to an application." },
+	{ "context", MISDN_CFG_CONTEXT, MISDN_CTYPE_STR, "default", NONE,
+		"Context to use for incoming calls." },
+	{ "language", MISDN_CFG_LANGUAGE, MISDN_CTYPE_STR, "en", NONE,
+		"Language." },
+	{ "musicclass", MISDN_CFG_MUSICCLASS, MISDN_CTYPE_STR, "default", NONE,
+		"Sets the musiconhold class." },
+	{ "callerid", MISDN_CFG_CALLERID, MISDN_CTYPE_STR, "", NONE,
+		"Sets the caller ID." },
+	{ "method", MISDN_CFG_METHOD, MISDN_CTYPE_STR, "standard", NONE,
+		"Sets the method to use for channel selection:\n"
+		"\t  standard    - always choose the first free channel with the lowest number\n"
+		"\t  round_robin - use the round robin algorithm to select a channel. use this\n"
+		"\t                if you want to balance your load." },
+	{ "dialplan", MISDN_CFG_DIALPLAN, MISDN_CTYPE_INT, "0", NONE,
+		"Dialplan means Type Of Number in ISDN Terms (for outgoing calls)\n"
+		"\n"
+		"\tThere are different types of the dialplan:\n"
+		"\n"
+		"\tdialplan -> outgoing Number\n"
+		"\tlocaldialplan -> callerid\n"
+		"\tcpndialplan -> connected party number\n"
+		"\n"
+		"\tdialplan options:\n"
+		"\n"
+		"\t0 - unknown\n"
+		"\t1 - International\n"
+		"\t2 - National\n"
+		"\t4 - Subscriber\n"
+		"\n"
+		"\tThis setting is used for outgoing calls." },
+	{ "localdialplan", MISDN_CFG_LOCALDIALPLAN, MISDN_CTYPE_INT, "0", NONE,
+		"Dialplan means Type Of Number in ISDN Terms (for outgoing calls)\n"
+		"\n"
+		"\tThere are different types of the dialplan:\n"
+		"\n"
+		"\tdialplan -> outgoing Number\n"
+		"\tlocaldialplan -> callerid\n"
+		"\tcpndialplan -> connected party number\n"
+		"\n"
+		"\tdialplan options:\n"
+		"\n"
+		"\t0 - unknown\n"
+		"\t1 - International\n"
+		"\t2 - National\n"
+		"\t4 - Subscriber\n"
+		"\n"
+		"\tThis setting is used for outgoing calls" },
+	{ "cpndialplan", MISDN_CFG_CPNDIALPLAN, MISDN_CTYPE_INT, "0", NONE,
+		"Dialplan means Type Of Number in ISDN Terms (for outgoing calls)\n"
+		"\n"
+		"\tThere are different types of the dialplan:\n"
+		"\n"
+		"\tdialplan -> outgoing Number\n"
+		"\tlocaldialplan -> callerid\n"
+		"\tcpndialplan -> connected party number\n"
+		"\n"
+		"\tdialplan options:\n"
+		"\n"
+		"\t0 - unknown\n"
+		"\t1 - International\n"
+		"\t2 - National\n"
+		"\t4 - Subscriber\n"
+		"\n"
+		"\tThis setting is used for outgoing calls." },
+	{ "nationalprefix", MISDN_CFG_NATPREFIX, MISDN_CTYPE_STR, "0", NONE,
+		"Prefix for national, this is put before the\n"
+		"\toad if an according dialplan is set by the other end." },
+	{ "internationalprefix", MISDN_CFG_INTERNATPREFIX, MISDN_CTYPE_STR, "00", NONE,
+		"Prefix for international, this is put before the\n"
+		"\toad if an according dialplan is set by the other end." },
+	{ "presentation", MISDN_CFG_PRES, MISDN_CTYPE_INT, "-1", NONE,
+		"These (presentation and screen) are the exact isdn screening and presentation\n"
+		"\tindicators.\n"
+		"\tIf -1 is given for both values, the presentation indicators are used from\n"
+		"\tAsterisks SetCallerPres application.\n"
+		"\n"
+		"\tscreen=0, presentation=0 -> callerid presented not screened\n"
+		"\tscreen=1, presentation=1 -> callerid presented but screened (the remote end doesn't see it!)" },
+	{ "screen", MISDN_CFG_SCREEN, MISDN_CTYPE_INT, "-1", NONE,
+		"These (presentation and screen) are the exact isdn screening and presentation\n"
+		"\tindicators.\n"
+		"\tIf -1 is given for both values, the presentation indicators are used from\n"
+		"\tAsterisks SetCallerPres application.\n"
+		"\n"
+		"\tscreen=0, presentation=0 -> callerid presented not screened\n"
+		"\tscreen=1, presentation=1 -> callerid presented but screened (the remote end doesn't see it!)" },
+	{ "always_immediate", MISDN_CFG_ALWAYS_IMMEDIATE, MISDN_CTYPE_BOOL, "no", NONE,
+		"Enable this to get into the s dialplan-extension.\n"
+		"\tThere you can use DigitTimeout if you can't or don't want to use\n"
+		"\tisdn overlap dial.\n"
+		"\tNOTE: This will jump into the s extension for every exten!" },
+	{ "nodialtone", MISDN_CFG_NODIALTONE, MISDN_CTYPE_BOOL, "no", NONE,
+		"Enable this to prevent chan_misdn to generate the dialtone\n"
+		"\tThis makes only sense together with the always_immediate=yes option\n"
+		"\tto generate your own dialtone with Playtones or so."},
+	{ "immediate", MISDN_CFG_IMMEDIATE, MISDN_CTYPE_BOOL, "no", NONE,
+		"Enable this if you want callers which called exactly the base\n"
+		"\tnumber (so no extension is set) to jump into the s extension.\n"
+		"\tIf the user dials something more, it jumps to the correct extension\n"
+		"\tinstead." },
+	{ "senddtmf", MISDN_CFG_SENDDTMF, MISDN_CTYPE_BOOL, "no", NONE,
+		"Enable this if we should produce DTMF Tones ourselves." },
+	{ "hold_allowed", MISDN_CFG_HOLD_ALLOWED, MISDN_CTYPE_BOOL, "no", NONE,
+		"Enable this to have support for hold and retrieve." },
+	{ "early_bconnect", MISDN_CFG_EARLY_BCONNECT, MISDN_CTYPE_BOOL, "yes", NONE,
+		"Disable this if you don't mind correct handling of Progress Indicators." },
+	{ "incoming_early_audio", MISDN_CFG_INCOMING_EARLY_AUDIO, MISDN_CTYPE_BOOL, "no", NONE,
+		"Turn this on if you like to send Tone Indications to a Incoming\n"
+		"\tisdn channel on a TE Port. Rarely used, only if the Telco allows\n"
+		"\tyou to send indications by yourself, normally the Telco sends the\n"
+		"\tindications to the remote party." },
+	{ "echocancel", MISDN_CFG_ECHOCANCEL, MISDN_CTYPE_BOOLINT, "0", 128,
+		"This enables echocancellation, with the given number of taps.\n"
+		"\tBe aware, move this setting only to outgoing portgroups!\n"
+		"\tA value of zero turns echocancellation off.\n"
+		"\n"
+		"\tPossible values are: 0,32,64,128,256,yes(=128),no(=0)" },
 #ifdef MISDN_1_2
-	{ "pipeline", MISDN_CFG_PIPELINE, MISDN_CTYPE_STR, NO_DEFAULT, NONE },
+	{ "pipeline", MISDN_CFG_PIPELINE, MISDN_CTYPE_STR, NO_DEFAULT, NONE,
+		"Set the configuration string for the mISDN dsp pipeline.\n"
+		"\n"
+		"\tExample for enabling the mg2 echo cancellation module with deftaps\n"
+		"\tset to 128:\n"
+		"\t\tmg2ec(deftaps=128)" },
 #endif
-	{ "need_more_infos", MISDN_CFG_NEED_MORE_INFOS, MISDN_CTYPE_BOOL, "0", NONE },
-	{ "noautorespond_on_setup", MISDN_CFG_NOAUTORESPOND_ON_SETUP, MISDN_CTYPE_BOOL, "0", NONE },
-	{ "overlapdial", MISDN_CFG_OVERLAP_DIAL, MISDN_CTYPE_BOOLINT, "0", 4 },
-	{ "nttimeout", MISDN_CFG_NTTIMEOUT, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "bridging", MISDN_CFG_BRIDGING, MISDN_CTYPE_BOOL, "yes", NONE },
-	{ "jitterbuffer", MISDN_CFG_JITTERBUFFER, MISDN_CTYPE_INT, "4000", NONE },
-	{ "jitterbuffer_upper_threshold", MISDN_CFG_JITTERBUFFER_UPPER_THRESHOLD, MISDN_CTYPE_INT, "0", NONE },
-	{ "callgroup", MISDN_CFG_CALLGROUP, MISDN_CTYPE_ASTGROUP, NO_DEFAULT, NONE },
-	{ "pickupgroup", MISDN_CFG_PICKUPGROUP, MISDN_CTYPE_ASTGROUP, NO_DEFAULT, NONE },
-	{ "msns", MISDN_CFG_MSNS, MISDN_CTYPE_MSNLIST, "*", NONE }
+#ifdef WITH_BEROEC
+	{ "bnechocancel", MISDN_CFG_BNECHOCANCEL, MISDN_CTYPE_BOOLINT, "yes", 64,
+		"echotail in ms (1-200)\n"},
+	{ "bnec_antihowl", MISDN_CFG_BNEC_ANTIHOWL, MISDN_CTYPE_INT, "0", NONE,
+		"Use antihowl\n"},
+	{ "bnec_nlp", MISDN_CFG_BNEC_NLP, MISDN_CTYPE_BOOL, "yes", NONE,
+		"Nonlinear Processing (much faster adaption)"},
+	{ "bnec_zerocoeff", MISDN_CFG_BNEC_ZEROCOEFF, MISDN_CTYPE_BOOL, "no", NONE,
+		"ZeroCoeffeciens\n"},
+	{ "bnec_tonedisabler", MISDN_CFG_BNEC_TD, MISDN_CTYPE_BOOL, "no", NONE,
+		"Disable Tone\n"},
+	{ "bnec_adaption", MISDN_CFG_BNEC_ADAPT, MISDN_CTYPE_INT, "1", NONE,
+		"Adaption mode (0=no,1=full,2=fast)\n"},
+#endif
+	{ "need_more_infos", MISDN_CFG_NEED_MORE_INFOS, MISDN_CTYPE_BOOL, "0", NONE,
+		"Send Setup_Acknowledge on incoming calls anyway (instead of PROCEEDING),\n"
+		"\tthis requests additional Infos, so we can waitfordigits without much\n"
+		"\tissues. This works only for PTP Ports" },
+	{ "noautorespond_on_setup", MISDN_CFG_NOAUTORESPOND_ON_SETUP, MISDN_CTYPE_BOOL, "0", NONE,
+		"Do not send SETUP_ACKNOWLEDGE or PROCEEDING automatically to the calling Party.\n"
+		"Instead we directly jump into the dialplan. This might be useful for fast call\n"
+		"rejection, or for some broken switches, that need hangup causes like busy in the.\n"
+		"RELEASE_COMPLETE Message, instead of the DISCONNECT Message.\n"},
+	{ "jitterbuffer", MISDN_CFG_JITTERBUFFER, MISDN_CTYPE_INT, "4000", NONE,
+		"The jitterbuffer." },
+	{ "jitterbuffer_upper_threshold", MISDN_CFG_JITTERBUFFER_UPPER_THRESHOLD, MISDN_CTYPE_INT, "0", NONE,
+		"Change this threshold to enable dejitter functionality." },
+	{ "callgroup", MISDN_CFG_CALLGROUP, MISDN_CTYPE_ASTGROUP, NO_DEFAULT, NONE,
+		"Callgroup." },
+	{ "pickupgroup", MISDN_CFG_PICKUPGROUP, MISDN_CTYPE_ASTGROUP, NO_DEFAULT, NONE,
+		"Pickupgroup." },
+	{ "max_incoming", MISDN_CFG_MAX_IN, MISDN_CTYPE_INT, "-1", NONE,
+		"Defines the maximum amount of incoming calls per port for this group.\n"
+		"\tCalls which exceed the maximum will be marked with the channel varible\n"
+		"\tMAX_OVERFLOW. It will contain the amount of overflowed calls" },
+	{ "max_outgoing", MISDN_CFG_MAX_OUT, MISDN_CTYPE_INT, "-1", NONE,
+		"Defines the maximum amount of outgoing calls per port for this group\n"
+		"\texceeding calls will be rejected" },
+
+	{ "reject_cause", MISDN_CFG_REJECT_CAUSE, MISDN_CTYPE_INT, "21", NONE,
+		"Defines the cause with which a 3. call is rejected on PTMP BRI."},
+	{ "faxdetect", MISDN_CFG_FAXDETECT, MISDN_CTYPE_STR, "no", NONE,
+		"Setup fax detection:\n"
+		"\t    no        - no fax detection\n"
+		"\t    incoming  - fax detection for incoming calls\n"
+		"\t    outgoing  - fax detection for outgoing calls\n"
+		"\t    both      - fax detection for incoming and outgoing calls\n"
+		"\tAdd +nojump to your value (i.e. faxdetect=both+nojump) if you don't want to jump into the\n"
+		"\tfax-extension but still want to detect the fax and prepare the channel for fax transfer." },
+	{ "faxdetect_timeout", MISDN_CFG_FAXDETECT_TIMEOUT, MISDN_CTYPE_INT, "5", NONE,
+		"Number of seconds the fax detection should do its job. After the given period of time,\n"
+		"\twe assume that it's not a fax call and save some CPU time by turning off fax detection.\n"
+		"\tSet this to 0 if you don't want a timeout (never stop detecting)." },
+	{ "faxdetect_context", MISDN_CFG_FAXDETECT_CONTEXT, MISDN_CTYPE_STR, NO_DEFAULT, NONE,
+		"Context to jump into if we detect a fax. Don't set this if you want to stay in the current context." },
+	{ "l1watcher_timeout", MISDN_CFG_L1_TIMEOUT, MISDN_CTYPE_BOOLINT, "0", 4,
+		"Watches the layer 1. If the layer 1 is down, it tries to\n"
+		"\tget it up. The timeout is given in seconds. with 0 as value it\n"
+		"\tdoes not watch the l1 at all\n"
+		"\n"
+		"\tThis option is only read at loading time of chan_misdn, which\n"
+		"\tmeans you need to unload and load chan_misdn to change the value,\n"
+		"\tan Asterisk restart should do the trick." },
+	{ "overlapdial", MISDN_CFG_OVERLAP_DIAL, MISDN_CTYPE_BOOLINT, "0", 4,
+		"Enables overlap dial for the given amount of seconds.\n"
+		"\tPossible values are positive integers or:\n"
+		"\t   yes (= 4 seconds)\n"
+		"\t   no  (= 0 seconds = disabled)" },
+	{ "nttimeout", MISDN_CFG_NTTIMEOUT, MISDN_CTYPE_BOOL, "no", NONE ,
+		"Set this to yes if you want calls disconnected in overlap mode" 
+		"when a timeout happens.\n"},
+	{ "bridging", MISDN_CFG_BRIDGING, MISDN_CTYPE_BOOL, "yes", NONE,
+	 	"Set this to yes/no, default is yes.\n"
+		"This can be used to have bridging enabled in general and to\n"
+		"disable it for specific ports. It makes sense to disable\n"
+		"bridging on NT Port where you plan to use the HOLD/RETRIEVE\n"
+		"features with ISDN phones.\n"
+		},
+	{ "msns", MISDN_CFG_MSNS, MISDN_CTYPE_MSNLIST, "*", NONE,
+		"MSN's for TE ports, listen on those numbers on the above ports, and\n"
+		"\tindicate the incoming calls to Asterisk.\n"
+		"\tHere you can give a comma seperated list, or simply an '*' for any msn." },
 };
 
 static const struct misdn_cfg_spec gen_spec[] = {
-	{ "debug", MISDN_GEN_DEBUG, MISDN_CTYPE_INT, "0", NONE },
+	{ "debug", MISDN_GEN_DEBUG, MISDN_CTYPE_INT, "0", NONE,
+		"Sets the debugging flag:\n"
+		"\t0 - No Debug\n"
+		"\t1 - mISDN Messages and * - Messages, and * - State changes\n"
+		"\t2 - Messages + Message specific Informations (e.g. bearer capability)\n"
+		"\t3 - very Verbose, the above + lots of Driver specific infos\n"
+		"\t4 - even more Verbose than 3" },
 #ifndef MISDN_1_2
-	{ "misdn_init", MISDN_GEN_MISDN_INIT, MISDN_CTYPE_STR, "/etc/misdn-init.conf", NONE },
+	{ "misdn_init", MISDN_GEN_MISDN_INIT, MISDN_CTYPE_STR, "/etc/misdn-init.conf", NONE,
+		"Set the path to the misdn-init.conf (for nt_ptp mode checking)." },
 #endif
-	{ "tracefile", MISDN_GEN_TRACEFILE, MISDN_CTYPE_STR, "/var/log/asterisk/misdn.log", NONE },
-	{ "bridging", MISDN_GEN_BRIDGING, MISDN_CTYPE_BOOL, "yes", NONE },
-	{ "stop_tone_after_first_digit", MISDN_GEN_STOP_TONE, MISDN_CTYPE_BOOL, "yes", NONE },
-	{ "append_digits2exten", MISDN_GEN_APPEND_DIGITS2EXTEN, MISDN_CTYPE_BOOL, "yes", NONE },
-	{ "dynamic_crypt", MISDN_GEN_DYNAMIC_CRYPT, MISDN_CTYPE_BOOL, "no", NONE },
-	{ "crypt_prefix", MISDN_GEN_CRYPT_PREFIX, MISDN_CTYPE_STR, NO_DEFAULT, NONE },
-	{ "crypt_keys", MISDN_GEN_CRYPT_KEYS, MISDN_CTYPE_STR, NO_DEFAULT, NONE },
-	{ "ntdebugflags", MISDN_GEN_NTDEBUGFLAGS, MISDN_CTYPE_INT, "0", NONE },
-	{ "ntdebugfile", MISDN_GEN_NTDEBUGFILE, MISDN_CTYPE_STR, "/var/log/misdn-nt.log", NONE }
+	{ "tracefile", MISDN_GEN_TRACEFILE, MISDN_CTYPE_STR, "/var/log/asterisk/misdn.log", NONE,
+		"Set the path to the massively growing trace file, if you want that." },
+	{ "bridging", MISDN_GEN_BRIDGING, MISDN_CTYPE_BOOL, "yes", NONE,
+		"Set this to yes if you want mISDN_dsp to bridge the calls in HW." },
+	{ "stop_tone_after_first_digit", MISDN_GEN_STOP_TONE, MISDN_CTYPE_BOOL, "yes", NONE,
+		"Stops dialtone after getting first digit on NT Port." },
+	{ "append_digits2exten", MISDN_GEN_APPEND_DIGITS2EXTEN, MISDN_CTYPE_BOOL, "yes", NONE,
+		"Wether to append overlapdialed Digits to Extension or not." },
+	{ "dynamic_crypt", MISDN_GEN_DYNAMIC_CRYPT, MISDN_CTYPE_BOOL, "no", NONE,
+		"Wether to look out for dynamic crypting attempts." },
+	{ "crypt_prefix", MISDN_GEN_CRYPT_PREFIX, MISDN_CTYPE_STR, NO_DEFAULT, NONE,
+		"What is used for crypting Protocol." },
+	{ "crypt_keys", MISDN_GEN_CRYPT_KEYS, MISDN_CTYPE_STR, NO_DEFAULT, NONE,
+		"Keys for cryption, you reference them in the dialplan\n"
+		"\tLater also in dynamic encr." },
+	{ "ntdebugflags", MISDN_GEN_NTDEBUGFLAGS, MISDN_CTYPE_INT, "0", NONE,
+	  	"No description yet."},
+	{ "ntdebugfile", MISDN_GEN_NTDEBUGFILE, MISDN_CTYPE_STR, "/var/log/misdn-nt.log", NONE,
+	  	"No description yet." }
 };
 
+
 /* array of port configs, default is at position 0. */
 static union misdn_cfg_pt **port_cfg;
 /* max number of available ports, is set on init */
@@ -162,28 +379,41 @@ static ast_mutex_t config_mutex;
 		"Please edit your misdn.conf and then do a \"misdn reload\".\n", name, value, section); \
 })
 
-static void _enum_array_map (void)
+static int _enum_array_map (void)
 {
-	int i, j;
+	int i, j, ok;
 
 	for (i = MISDN_CFG_FIRST + 1; i < MISDN_CFG_LAST; ++i) {
 		if (i == MISDN_CFG_PTP)
 			continue;
+		ok = 0;
 		for (j = 0; j < NUM_PORT_ELEMENTS; ++j) {
 			if (port_spec[j].elem == i) {
 				map[i] = j;
+				ok = 1;
 				break;
 			}
 		}
+		if (!ok) {
+			ast_log(LOG_WARNING, "Enum element %d in misdn_cfg_elements (port section) has no corresponding element in the config struct!\n", i);
+			return -1;
+		}
 	}
 	for (i = MISDN_GEN_FIRST + 1; i < MISDN_GEN_LAST; ++i) {
+		ok = 0;
 		for (j = 0; j < NUM_GEN_ELEMENTS; ++j) {
 			if (gen_spec[j].elem == i) {
 				map[i] = j;
+				ok = 1;
 				break;
 			}
 		}
+		if (!ok) {
+			ast_log(LOG_WARNING, "Enum element %d in misdn_cfg_elements (general section) has no corresponding element in the config struct!\n", i);
+			return -1;
+		}
 	}
+	return 0;
 }
 
 static int get_cfg_position (char *name, int type)
@@ -324,6 +554,87 @@ void misdn_cfg_get (int port, enum misdn_cfg_elements elem, void *buf, int bufsi
 	misdn_cfg_unlock();
 }
 
+enum misdn_cfg_elements misdn_cfg_get_elem (char *name)
+{
+	int pos;
+
+	/* here comes a hack to replace the (not existing) "name" elemet with the "ports" element */
+	if (!strcmp(name, "ports"))
+		return MISDN_CFG_GROUPNAME;
+	if (!strcmp(name, "name"))
+		return MISDN_CFG_FIRST;
+
+	pos = get_cfg_position (name, PORT_CFG);
+	if (pos >= 0)
+		return port_spec[pos].elem;
+	
+	pos = get_cfg_position (name, GEN_CFG);
+	if (pos >= 0)
+		return gen_spec[pos].elem;
+	
+	return MISDN_CFG_FIRST;
+}
+
+void misdn_cfg_get_name (enum misdn_cfg_elements elem, void *buf, int bufsize)
+{
+	struct misdn_cfg_spec *spec = NULL;
+	int place = map[elem];
+
+	/* the ptp hack */
+	if (elem == MISDN_CFG_PTP) {
+		memset(buf, 0, 1);
+		return;
+	}
+	
+	/* here comes a hack to replace the (not existing) "name" elemet with the "ports" element */
+	if (elem == MISDN_CFG_GROUPNAME) {
+		if (!snprintf(buf, bufsize, "ports"))
+			memset(buf, 0, 1);
+		return;
+	}
+	
+	if ((elem > MISDN_CFG_FIRST) && (elem < MISDN_CFG_LAST))
+		spec = (struct misdn_cfg_spec *)port_spec;
+	else if ((elem > MISDN_GEN_FIRST) && (elem < MISDN_GEN_LAST))
+		spec = (struct misdn_cfg_spec *)gen_spec;
+
+	if (!spec || !memccpy(buf, spec[place].name, 0, bufsize))
+		memset(buf, 0, 1);
+}
+
+void misdn_cfg_get_desc (enum misdn_cfg_elements elem, void *buf, int bufsize, void *buf_default, int bufsize_default)
+{
+	int place = map[elem];
+	struct misdn_cfg_spec *spec = NULL;
+
+	/* here comes a hack to replace the (not existing) "name" elemet with the "ports" element */
+	if (elem == MISDN_CFG_GROUPNAME) {
+		if (!memccpy(buf, ports_description, 0, bufsize))
+			memset(buf, 0, 1);
+		if (buf_default && bufsize_default)
+			memset(buf_default, 0, 1);
+		return;
+	}
+
+	if ((elem > MISDN_CFG_FIRST) && (elem < MISDN_CFG_LAST))
+		spec = (struct misdn_cfg_spec *)port_spec;
+	else if ((elem > MISDN_GEN_FIRST) && (elem < MISDN_GEN_LAST))
+		spec = (struct misdn_cfg_spec *)gen_spec;
+		
+	if (!spec || !spec[place].desc)
+		memset(buf, 0, 1);
+	else {
+		if (!memccpy(buf, spec[place].desc, 0, bufsize))
+			memset(buf, 0, 1);
+		if (buf_default && bufsize) {
+			if (!strcmp(spec[place].def, NO_DEFAULT))
+				memset(buf_default, 0, 1);
+			else if (!memccpy(buf_default, spec[place].def, 0, bufsize_default))
+				memset(buf_default, 0, 1);
+		}
+	}
+}
+
 int misdn_cfg_is_msn_valid (int port, char* msn)
 {
 	int re = 0;
@@ -351,7 +662,9 @@ int misdn_cfg_is_msn_valid (int port, char* msn)
 
 int misdn_cfg_is_port_valid (int port)
 {
-	return (port >= 1 && port <= max_ports);
+	int gn = map[MISDN_CFG_GROUPNAME];
+
+	return (port >= 1 && port <= max_ports && port_cfg[port][gn].str);
 }
 
 int misdn_cfg_is_group_method (char *group, enum misdn_cfg_method meth)
@@ -771,15 +1084,20 @@ int misdn_cfg_init (int this_max_ports)
 	struct ast_variable *v;
 
 	if (!(cfg = AST_LOAD_CFG(config))) {
-		ast_log(LOG_WARNING,"no misdn.conf ?\n");
+		ast_log(LOG_WARNING, "missing file: misdn.conf\n");
 		return -1;
 	}
 
+	ast_mutex_init(&config_mutex);
+
 	misdn_cfg_lock();
 
 	if (this_max_ports) {
 		/* this is the first run */
 		max_ports = this_max_ports;
+		map = (int *)calloc(MISDN_GEN_LAST + 1, sizeof(int));
+		if (_enum_array_map())
+			return -1;
 		p = (char *)calloc(1, (max_ports + 1) * sizeof(union misdn_cfg_pt *)
 						   + (max_ports + 1) * NUM_PORT_ELEMENTS * sizeof(union misdn_cfg_pt));
 		port_cfg = (union misdn_cfg_pt **)p;
@@ -790,8 +1108,6 @@ int misdn_cfg_init (int this_max_ports)
 		}
 		general_cfg = (union misdn_cfg_pt *)calloc(1, sizeof(union misdn_cfg_pt *) * NUM_GEN_ELEMENTS);
 		ptp = (int *)calloc(max_ports + 1, sizeof(int));
-		map = (int *)calloc(MISDN_GEN_LAST + 1, sizeof(int));
-		_enum_array_map();
 	}
 	else {
 		/* misdn reload */
@@ -806,12 +1122,12 @@ int misdn_cfg_init (int this_max_ports)
 
 	while(cat) {
 		v = ast_variable_browse(cfg, cat);
-		if (!strcasecmp(cat,"general")) {
+		if (!strcasecmp(cat, "general")) {
 			_build_general_config(v);
 		} else {
 			_build_port_config(v, cat);
 		}
-		cat = ast_category_browse(cfg,cat);
+		cat = ast_category_browse(cfg, cat);
 	}
 
 	_fill_defaults();
@@ -821,3 +1137,5 @@ int misdn_cfg_init (int this_max_ports)
 
 	return 0;
 }
+
+
