@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 1999 - 2005, Digium, Inc.
+ * Copyright (C) 1999 - 2006, Digium, Inc.
  * Portions Copyright (C) 2005, Anthony Minessale II
  *
  * See http://www.asterisk.org for more information about
@@ -19,59 +19,63 @@
  * 
  * \brief Conditional logic dialplan functions
  * 
+ * \author Anthony Minessale II
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 44808 $")
+
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
-#include "asterisk.h"
-
-/* ASTERISK_FILE_VERSION(__FILE__, "$Revision: 27051 $") */
-
+#include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
 #include "asterisk/logger.h"
 #include "asterisk/utils.h"
 #include "asterisk/app.h"
-#include "asterisk/config.h"		/* for ast_true */
 
-static char *builtin_function_isnull(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
+static int isnull(struct ast_channel *chan, char *cmd, char *data,
+		  char *buf, size_t len)
 {
-	return data && *data ? "0" : "1";
+	strcpy(buf, data && *data ? "0" : "1");
+
+	return 0;
 }
 
-static char *builtin_function_exists(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
+static int exists(struct ast_channel *chan, char *cmd, char *data, char *buf,
+		  size_t len)
 {
-	return data && *data ? "1" : "0";
+	strcpy(buf, data && *data ? "1" : "0");
+
+	return 0;
 }
 
-static char *builtin_function_iftime(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
+static int iftime(struct ast_channel *chan, char *cmd, char *data, char *buf,
+		  size_t len)
 {
 	struct ast_timing timing;
-	char *ret;
 	char *expr;
 	char *iftrue;
 	char *iffalse;
 
-	if (!(data = ast_strdupa(data))) {
-		ast_log(LOG_WARNING, "Memory Error!\n");
-		return NULL;
-	}
-
 	data = ast_strip_quoted(data, "\"", "\"");
 	expr = strsep(&data, "?");
 	iftrue = strsep(&data, ":");
 	iffalse = data;
 
 	if (ast_strlen_zero(expr) || !(iftrue || iffalse)) {
-		ast_log(LOG_WARNING, "Syntax IFTIME(<timespec>?[<true>][:<false>])\n");
-		return NULL;
+		ast_log(LOG_WARNING,
+				"Syntax IFTIME(<timespec>?[<true>][:<false>])\n");
+		return -1;
 	}
 
 	if (!ast_build_timing(&timing, expr)) {
 		ast_log(LOG_WARNING, "Invalid Time Spec.\n");
-		return NULL;
+		return -1;
 	}
 
 	if (iftrue)
@@ -79,26 +83,18 @@ static char *builtin_function_iftime(struct ast_channel *chan, char *cmd, char *
 	if (iffalse)
 		iffalse = ast_strip_quoted(iffalse, "\"", "\"");
 
-	if ((ret = ast_check_timing(&timing) ? iftrue : iffalse)) {
-		ast_copy_string(buf, ret, len);
-		ret = buf;
-	} 
-	
-	return ret;
+	ast_copy_string(buf, ast_check_timing(&timing) ? iftrue : iffalse, len);
+
+	return 0;
 }
 
-static char *builtin_function_if(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
+static int acf_if(struct ast_channel *chan, char *cmd, char *data, char *buf,
+		  size_t len)
 {
-	char *ret;
 	char *expr;
 	char *iftrue;
 	char *iffalse;
 
-	if (!(data = ast_strdupa(data))) {
-		ast_log(LOG_WARNING, "Memory Error!\n");
-		return NULL;
-	}
-
 	data = ast_strip_quoted(data, "\"", "\"");
 	expr = strsep(&data, "?");
 	iftrue = strsep(&data, ":");
@@ -106,7 +102,7 @@ static char *builtin_function_if(struct ast_channel *chan, char *cmd, char *data
 
 	if (ast_strlen_zero(expr) || !(iftrue || iffalse)) {
 		ast_log(LOG_WARNING, "Syntax IF(<expr>?[<true>][:<false>])\n");
-		return NULL;
+		return -1;
 	}
 
 	expr = ast_strip(expr);
@@ -115,30 +111,23 @@ static char *builtin_function_if(struct ast_channel *chan, char *cmd, char *data
 	if (iffalse)
 		iffalse = ast_strip_quoted(iffalse, "\"", "\"");
 
-	if ((ret = pbx_checkcondition(expr) ? iftrue : iffalse)) {
-		ast_copy_string(buf, ret, len);
-		ret = buf;
-	} 
-	
-	return ret;
+	ast_copy_string(buf, pbx_checkcondition(expr) ? (S_OR(iftrue, "")) : (S_OR(iffalse, "")), len);
+
+	return 0;
 }
 
-static char *builtin_function_set(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
+static int set(struct ast_channel *chan, char *cmd, char *data, char *buf,
+	       size_t len)
 {
 	char *varname;
 	char *val;
 
-	if (!(data = ast_strdupa(data))) {
-		ast_log(LOG_WARNING, "Memory Error!\n");
-		return NULL;
-	}
-
 	varname = strsep(&data, "=");
 	val = data;
 
 	if (ast_strlen_zero(varname) || !val) {
 		ast_log(LOG_WARNING, "Syntax SET(<varname>=[<value>])\n");
-		return NULL;
+		return -1;
 	}
 
 	varname = ast_strip(varname);
@@ -146,56 +135,70 @@ static char *builtin_function_set(struct ast_channel *chan, char *cmd, char *dat
 	pbx_builtin_setvar_helper(chan, varname, val);
 	ast_copy_string(buf, val, len);
 
-	return buf;
+	return 0;
 }
 
-#ifndef BUILTIN_FUNC
-static
-#endif
-struct ast_custom_function isnull_function = {
+static struct ast_custom_function isnull_function = {
 	.name = "ISNULL",
 	.synopsis = "NULL Test: Returns 1 if NULL or 0 otherwise",
 	.syntax = "ISNULL(<data>)",
-	.read = builtin_function_isnull,
+	.read = isnull,
 };
 
-#ifndef BUILTIN_FUNC
-static
-#endif
-struct ast_custom_function set_function = {
+static struct ast_custom_function set_function = {
 	.name = "SET",
 	.synopsis = "SET assigns a value to a channel variable",
 	.syntax = "SET(<varname>=[<value>])",
-	.read = builtin_function_set,
+	.read = set,
 };
 
-#ifndef BUILTIN_FUNC
-static
-#endif
-struct ast_custom_function exists_function = {
+static struct ast_custom_function exists_function = {
 	.name = "EXISTS",
 	.synopsis = "Existence Test: Returns 1 if exists, 0 otherwise",
 	.syntax = "EXISTS(<data>)",
-	.read = builtin_function_exists,
+	.read = exists,
 };
 
-#ifndef BUILTIN_FUNC
-static
-#endif
-struct ast_custom_function if_function = {
+static struct ast_custom_function if_function = {
 	.name = "IF",
-	.synopsis = "Conditional: Returns the data following '?' if true else the data following ':'",
+	.synopsis =
+		"Conditional: Returns the data following '?' if true else the data following ':'",
 	.syntax = "IF(<expr>?[<true>][:<false>])",
-	.read = builtin_function_if,
+	.read = acf_if,
 };
 
-
-#ifndef BUILTIN_FUNC
-static
-#endif
-struct ast_custom_function if_time_function = {
+static struct ast_custom_function if_time_function = {
 	.name = "IFTIME",
-	.synopsis = "Temporal Conditional: Returns the data following '?' if true else the data following ':'",
+	.synopsis =
+		"Temporal Conditional: Returns the data following '?' if true else the data following ':'",
 	.syntax = "IFTIME(<timespec>?[<true>][:<false>])",
-	.read = builtin_function_iftime,
+	.read = iftime,
 };
+
+static int unload_module(void)
+{
+	int res = 0;
+
+	res |= ast_custom_function_unregister(&isnull_function);
+	res |= ast_custom_function_unregister(&set_function);
+	res |= ast_custom_function_unregister(&exists_function);
+	res |= ast_custom_function_unregister(&if_function);
+	res |= ast_custom_function_unregister(&if_time_function);
+
+	return res;
+}
+
+static int load_module(void)
+{
+	int res = 0;
+
+	res |= ast_custom_function_register(&isnull_function);
+	res |= ast_custom_function_register(&set_function);
+	res |= ast_custom_function_register(&exists_function);
+	res |= ast_custom_function_register(&if_function);
+	res |= ast_custom_function_register(&if_time_function);
+
+	return res;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Logical dialplan functions");
