@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 1999 - 2005, Digium, Inc.
+ * Copyright (C) 1999 - 2006, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
@@ -49,7 +49,7 @@ enum chanspy_flags {
 };
 
 struct ast_channel_spy_queue {
-	struct ast_frame *head;
+	AST_LIST_HEAD_NOLOCK(, ast_frame) list;
 	unsigned int samples;
 	unsigned int format;
 };
