@@ -21,9 +21,19 @@
 /*! \file
  *
  * \brief ADSI support 
- * 
+ *
+ * \author Mark Spencer <markster@digium.com> 
+ *
+ * \note this module is required by app_voicemail and app_getcpeid
+ * \todo Move app_getcpeid into this module
+ * \todo Create a core layer so that app_voicemail does not require
+ * 	res_adsi to load
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 53780 $")
+
 #include <time.h>
 #include <string.h>
 #include <stdio.h>
@@ -32,10 +42,6 @@
 #include <math.h>
 #include <errno.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
-
 #include "asterisk/ulaw.h"
 #include "asterisk/alaw.h"
 #include "asterisk/callerid.h"
@@ -316,7 +322,7 @@ static int __adsi_transmit_messages(struct ast_channel *chan, unsigned char **ms
 	
 }
 
-int adsi_begin_download(struct ast_channel *chan, char *service, unsigned char *fdn, unsigned char *sec, int version)
+int ast_adsi_begin_download(struct ast_channel *chan, char *service, unsigned char *fdn, unsigned char *sec, int version)
 {
 	int bytes;
 	unsigned char buf[256];
@@ -324,8 +330,8 @@ int adsi_begin_download(struct ast_channel *chan, char *service, unsigned char *
 	bytes = 0;
 	/* Setup the resident soft key stuff, a piece at a time */
 	/* Upload what scripts we can for voicemail ahead of time */
-	bytes += adsi_download_connect(buf + bytes, service, fdn, sec, version);
-	if (adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DOWNLOAD, 0))
+	bytes += ast_adsi_download_connect(buf + bytes, service, fdn, sec, version);
+	if (ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DOWNLOAD, 0))
 		return -1;
 	if (ast_readstring(chan, ack, 1, 10000, 10000, ""))
 		return -1;
@@ -335,20 +341,20 @@ int adsi_begin_download(struct ast_channel *chan, char *service, unsigned char *
 	return -1;
 }
 
-int adsi_end_download(struct ast_channel *chan)
+int ast_adsi_end_download(struct ast_channel *chan)
 {
 	int bytes;
 	unsigned char buf[256];
         bytes = 0;
         /* Setup the resident soft key stuff, a piece at a time */
         /* Upload what scripts we can for voicemail ahead of time */
-        bytes += adsi_download_disconnect(buf + bytes);
-	if (adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DOWNLOAD, 0))
+        bytes += ast_adsi_download_disconnect(buf + bytes);
+	if (ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DOWNLOAD, 0))
 		return -1;
 	return 0;
 }
 
-int adsi_transmit_message_full(struct ast_channel *chan, unsigned char *msg, int msglen, int msgtype, int dowait)
+int ast_adsi_transmit_message_full(struct ast_channel *chan, unsigned char *msg, int msglen, int msgtype, int dowait)
 {
 	unsigned char *msgs[5] = { NULL, NULL, NULL, NULL, NULL };
 	int msglens[5];
@@ -422,12 +428,12 @@ int adsi_transmit_message_full(struct ast_channel *chan, unsigned char *msg, int
 	return res;
 }
 
-int adsi_transmit_message(struct ast_channel *chan, unsigned char *msg, int msglen, int msgtype)
+int ast_adsi_transmit_message(struct ast_channel *chan, unsigned char *msg, int msglen, int msgtype)
 {
-	return adsi_transmit_message_full(chan, msg, msglen, msgtype, 1);
+	return ast_adsi_transmit_message_full(chan, msg, msglen, msgtype, 1);
 }
 
-static inline int ccopy(unsigned char *dst, unsigned char *src, int max)
+static inline int ccopy(unsigned char *dst, const unsigned char *src, int max)
 {
 	int x=0;
 	/* Carefully copy the requested data */
@@ -438,7 +444,7 @@ static inline int ccopy(unsigned char *dst, unsigned char *src, int max)
 	return x;
 }
 
-int adsi_load_soft_key(unsigned char *buf, int key, char *llabel, char *slabel, char *ret, int data)
+int ast_adsi_load_soft_key(unsigned char *buf, int key, const char *llabel, const char *slabel, const char *ret, int data)
 {
 	int bytes=0;
 
@@ -452,13 +458,13 @@ int adsi_load_soft_key(unsigned char *buf, int key, char *llabel, char *slabel,
 	buf[bytes++] = key;
 
 	/* Carefully copy long label */
-	bytes += ccopy(buf + bytes, (unsigned char *)llabel, 18);
+	bytes += ccopy(buf + bytes, (const unsigned char *)llabel, 18);
 
 	/* Place delimiter */
 	buf[bytes++] = 0xff;
 
 	/* Short label */
-	bytes += ccopy(buf + bytes, (unsigned char *)slabel, 7);
+	bytes += ccopy(buf + bytes, (const unsigned char *)slabel, 7);
 
 
 	/* If specified, copy return string */
@@ -468,7 +474,7 @@ int adsi_load_soft_key(unsigned char *buf, int key, char *llabel, char *slabel,
 		if (data)
 			buf[bytes++] = ADSI_SWITCH_TO_DATA2;
 		/* Carefully copy return string */
-		bytes += ccopy(buf + bytes, (unsigned char *)ret, 20);
+		bytes += ccopy(buf + bytes, (const unsigned char *)ret, 20);
 
 	}
 	/* Replace parameter length */
@@ -477,7 +483,7 @@ int adsi_load_soft_key(unsigned char *buf, int key, char *llabel, char *slabel,
 	
 }
 
-int adsi_connect_session(unsigned char *buf, unsigned char *fdn, int ver)
+int ast_adsi_connect_session(unsigned char *buf, unsigned char *fdn, int ver)
 {
 	int bytes=0;
 	int x;
@@ -500,7 +506,7 @@ int adsi_connect_session(unsigned char *buf, unsigned char *fdn, int ver)
 
 }
 
-int adsi_download_connect(unsigned char *buf, char *service,  unsigned char *fdn, unsigned char *sec, int ver)
+int ast_adsi_download_connect(unsigned char *buf, char *service,  unsigned char *fdn, unsigned char *sec, int ver)
 {
 	int bytes=0;
 	int x;
@@ -530,7 +536,7 @@ int adsi_download_connect(unsigned char *buf, char *service,  unsigned char *fdn
 
 }
 
-int adsi_disconnect_session(unsigned char *buf)
+int ast_adsi_disconnect_session(unsigned char *buf)
 {
 	int bytes=0;
 
@@ -545,7 +551,7 @@ int adsi_disconnect_session(unsigned char *buf)
 
 }
 
-int adsi_query_cpeid(unsigned char *buf)
+int ast_adsi_query_cpeid(unsigned char *buf)
 {
 	int bytes = 0;
 	buf[bytes++] = ADSI_QUERY_CPEID;
@@ -555,7 +561,7 @@ int adsi_query_cpeid(unsigned char *buf)
 	return bytes;
 }
 
-int adsi_query_cpeinfo(unsigned char *buf)
+int ast_adsi_query_cpeinfo(unsigned char *buf)
 {
 	int bytes = 0;
 	buf[bytes++] = ADSI_QUERY_CONFIG;
@@ -565,7 +571,7 @@ int adsi_query_cpeinfo(unsigned char *buf)
 	return bytes;
 }
 
-int adsi_read_encoded_dtmf(struct ast_channel *chan, unsigned char *buf, int maxlen)
+int ast_adsi_read_encoded_dtmf(struct ast_channel *chan, unsigned char *buf, int maxlen)
 {
 	int bytes = 0;
 	int res;
@@ -600,21 +606,21 @@ int adsi_read_encoded_dtmf(struct ast_channel *chan, unsigned char *buf, int max
 	return bytes;
 }
 
-int adsi_get_cpeid(struct ast_channel *chan, unsigned char *cpeid, int voice)
+int ast_adsi_get_cpeid(struct ast_channel *chan, unsigned char *cpeid, int voice)
 {
 	unsigned char buf[256];
 	int bytes = 0;
 	int res;
-	bytes += adsi_data_mode(buf);
-	adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+	bytes += ast_adsi_data_mode(buf);
+	ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 
 	bytes = 0;
-	bytes += adsi_query_cpeid(buf);
-	adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+	bytes += ast_adsi_query_cpeid(buf);
+	ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 
 	/* Get response */
 	memset(buf, 0, sizeof(buf));
-	res = adsi_read_encoded_dtmf(chan, cpeid, 4);
+	res = ast_adsi_read_encoded_dtmf(chan, cpeid, 4);
 	if (res != 4) {
 		ast_log(LOG_WARNING, "Got %d bytes back of encoded DTMF, expecting 4\n", res);
 		res = 0;
@@ -624,25 +630,25 @@ int adsi_get_cpeid(struct ast_channel *chan, unsigned char *cpeid, int voice)
 
 	if (voice) {
 		bytes = 0;
-		bytes += adsi_voice_mode(buf, 0);
-		adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+		bytes += ast_adsi_voice_mode(buf, 0);
+		ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 		/* Ignore the resulting DTMF B announcing it's in voice mode */
 		ast_waitfordigit(chan, 1000);
 	}
 	return res;
 }
 
-int adsi_get_cpeinfo(struct ast_channel *chan, int *width, int *height, int *buttons, int voice)
+int ast_adsi_get_cpeinfo(struct ast_channel *chan, int *width, int *height, int *buttons, int voice)
 {
 	unsigned char buf[256];
 	int bytes = 0;
 	int res;
-	bytes += adsi_data_mode(buf);
-	adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+	bytes += ast_adsi_data_mode(buf);
+	ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 
 	bytes = 0;
-	bytes += adsi_query_cpeinfo(buf);
-	adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+	bytes += ast_adsi_query_cpeinfo(buf);
+	ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 
 	/* Get width */
 	memset(buf, 0, sizeof(buf));
@@ -689,15 +695,15 @@ int adsi_get_cpeinfo(struct ast_channel *chan, int *width, int *height, int *but
 	}
 	if (voice) {
 		bytes = 0;
-		bytes += adsi_voice_mode(buf, 0);
-		adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+		bytes += ast_adsi_voice_mode(buf, 0);
+		ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 		/* Ignore the resulting DTMF B announcing it's in voice mode */
 		ast_waitfordigit(chan, 1000);
 	}
 	return res;
 }
 
-int adsi_data_mode(unsigned char *buf)
+int ast_adsi_data_mode(unsigned char *buf)
 {
 	int bytes=0;
 
@@ -712,7 +718,7 @@ int adsi_data_mode(unsigned char *buf)
 
 }
 
-int adsi_clear_soft_keys(unsigned char *buf)
+int ast_adsi_clear_soft_keys(unsigned char *buf)
 {
 	int bytes=0;
 
@@ -727,7 +733,7 @@ int adsi_clear_soft_keys(unsigned char *buf)
 
 }
 
-int adsi_clear_screen(unsigned char *buf)
+int ast_adsi_clear_screen(unsigned char *buf)
 {
 	int bytes=0;
 
@@ -742,7 +748,7 @@ int adsi_clear_screen(unsigned char *buf)
 
 }
 
-int adsi_voice_mode(unsigned char *buf, int when)
+int ast_adsi_voice_mode(unsigned char *buf, int when)
 {
 	int bytes=0;
 
@@ -759,7 +765,7 @@ int adsi_voice_mode(unsigned char *buf, int when)
 
 }
 
-int adsi_available(struct ast_channel *chan)
+int ast_adsi_available(struct ast_channel *chan)
 {
 	int cpe = chan->adsicpe & 0xff;
 	if ((cpe == AST_ADSI_AVAILABLE) ||
@@ -768,7 +774,7 @@ int adsi_available(struct ast_channel *chan)
 	return 0;
 }
 
-int adsi_download_disconnect(unsigned char *buf)
+int ast_adsi_download_disconnect(unsigned char *buf)
 {
 	int bytes=0;
 
@@ -783,8 +789,8 @@ int adsi_download_disconnect(unsigned char *buf)
 
 }
 
-int adsi_display(unsigned char *buf, int page, int line, int just, int wrap, 
-		 char *col1, char *col2)
+int ast_adsi_display(unsigned char *buf, int page, int line, int just, int wrap, 
+		     char *col1, char *col2)
 {
 	int bytes=0;
 
@@ -829,7 +835,7 @@ int adsi_display(unsigned char *buf, int page, int line, int just, int wrap,
 
 }
 
-int adsi_input_control(unsigned char *buf, int page, int line, int display, int format, int just)
+int ast_adsi_input_control(unsigned char *buf, int page, int line, int display, int format, int just)
 {
 	int bytes=0;
 
@@ -852,7 +858,7 @@ int adsi_input_control(unsigned char *buf, int page, int line, int display, int
 
 }
 
-int adsi_input_format(unsigned char *buf, int num, int dir, int wrap, char *format1, char *format2)
+int ast_adsi_input_format(unsigned char *buf, int num, int dir, int wrap, char *format1, char *format2)
 {
 	int bytes = 0;
 
@@ -871,7 +877,7 @@ int adsi_input_format(unsigned char *buf, int num, int dir, int wrap, char *form
 	return bytes;
 }
 
-int adsi_set_keys(unsigned char *buf, unsigned char *keys)
+int ast_adsi_set_keys(unsigned char *buf, unsigned char *keys)
 {
 	int bytes=0;
 	int x;
@@ -886,7 +892,7 @@ int adsi_set_keys(unsigned char *buf, unsigned char *keys)
 	return bytes;
 }
 
-int adsi_set_line(unsigned char *buf, int page, int line)
+int ast_adsi_set_line(unsigned char *buf, int page, int line)
 {
 	int bytes=0;
 
@@ -917,7 +923,7 @@ int adsi_set_line(unsigned char *buf, int page, int line)
 static int total = 0;
 static int speeds = 0;
 
-int adsi_channel_restore(struct ast_channel *chan)
+int ast_adsi_channel_restore(struct ast_channel *chan)
 {
 	unsigned char dsp[256];
 	int bytes;
@@ -928,7 +934,7 @@ int adsi_channel_restore(struct ast_channel *chan)
 
 	/* Start with initial display setup */
 	bytes = 0;
-	bytes += adsi_set_line(dsp + bytes, ADSI_INFO_PAGE, 1);
+	bytes += ast_adsi_set_line(dsp + bytes, ADSI_INFO_PAGE, 1);
 
 	/* Prepare key setup messages */
 
@@ -937,26 +943,26 @@ int adsi_channel_restore(struct ast_channel *chan)
 		for (x=0;x<speeds;x++) {
 			keyd[x] = ADSI_SPEED_DIAL + x;
 		}
-		bytes += adsi_set_keys(dsp + bytes, keyd);
+		bytes += ast_adsi_set_keys(dsp + bytes, keyd);
 	}
-	adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0);
+	ast_adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0);
 	return 0;
 
 }
 
-int adsi_print(struct ast_channel *chan, char **lines, int *aligns, int voice)
+int ast_adsi_print(struct ast_channel *chan, char **lines, int *aligns, int voice)
 {
 	unsigned char buf[4096];
 	int bytes=0;
 	int res;
 	int x;
 	for(x=0;lines[x];x++) 
-		bytes += adsi_display(buf + bytes, ADSI_INFO_PAGE, x+1, aligns[x], 0, lines[x], "");
-	bytes += adsi_set_line(buf + bytes, ADSI_INFO_PAGE, 1);
+		bytes += ast_adsi_display(buf + bytes, ADSI_INFO_PAGE, x+1, aligns[x], 0, lines[x], "");
+	bytes += ast_adsi_set_line(buf + bytes, ADSI_INFO_PAGE, 1);
 	if (voice) {
-		bytes += adsi_voice_mode(buf + bytes, 0);
+		bytes += ast_adsi_voice_mode(buf + bytes, 0);
 	}
-	res = adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
+	res = ast_adsi_transmit_message_full(chan, buf, bytes, ADSI_MSG_DISPLAY, 0);
 	if (voice) {
 		/* Ignore the resulting DTMF B announcing it's in voice mode */
 		ast_waitfordigit(chan, 1000);
@@ -964,7 +970,7 @@ int adsi_print(struct ast_channel *chan, char **lines, int *aligns, int voice)
 	return res;
 }
 
-int adsi_load_session(struct ast_channel *chan, unsigned char *app, int ver, int data)
+int ast_adsi_load_session(struct ast_channel *chan, unsigned char *app, int ver, int data)
 {
 	unsigned char dsp[256];
 	int bytes;
@@ -975,13 +981,13 @@ int adsi_load_session(struct ast_channel *chan, unsigned char *app, int ver, int
 
 	/* Connect to session */
 	bytes = 0;
-	bytes += adsi_connect_session(dsp + bytes, app, ver);
+	bytes += ast_adsi_connect_session(dsp + bytes, app, ver);
 
 	if (data)
-		bytes += adsi_data_mode(dsp + bytes);
+		bytes += ast_adsi_data_mode(dsp + bytes);
 
 	/* Prepare key setup messages */
-	if (adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0))
+	if (ast_adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0))
 		return -1;
 	if (app) {
 		res = ast_readstring(chan, resp, 1, 1200, 1200, "");
@@ -1005,7 +1011,7 @@ int adsi_load_session(struct ast_channel *chan, unsigned char *app, int ver, int
 
 }
 
-int adsi_unload_session(struct ast_channel *chan)
+int ast_adsi_unload_session(struct ast_channel *chan)
 {
 	unsigned char dsp[256];
 	int bytes;
@@ -1014,11 +1020,11 @@ int adsi_unload_session(struct ast_channel *chan)
 
 	/* Connect to session */
 	bytes = 0;
-	bytes += adsi_disconnect_session(dsp + bytes);
-	bytes += adsi_voice_mode(dsp + bytes, 0);
+	bytes += ast_adsi_disconnect_session(dsp + bytes);
+	bytes += ast_adsi_voice_mode(dsp + bytes, 0);
 
 	/* Prepare key setup messages */
-	if (adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0))
+	if (ast_adsi_transmit_message_full(chan, dsp, bytes, ADSI_MSG_DISPLAY, 0))
 		return -1;
 	return 0;
 }
@@ -1041,9 +1047,9 @@ static void init_state(void)
 
 	for (x=0;x<ADSI_MAX_INTRO;x++)
 		aligns[x] = ADSI_JUST_CENT;
-	strncpy(intro[0], "Welcome to the", sizeof(intro[0]) - 1);
-	strncpy(intro[1], "Asterisk", sizeof(intro[1]) - 1);
-	strncpy(intro[2], "Open Source PBX", sizeof(intro[2]) - 1);
+	ast_copy_string(intro[0], "Welcome to the", sizeof(intro[0]));
+	ast_copy_string(intro[1], "Asterisk", sizeof(intro[1]));
+	ast_copy_string(intro[2], "Open Source PBX", sizeof(intro[2]));
 	total = 3;
 	speeds = 0;
 	for (x=3;x<ADSI_MAX_INTRO;x++)
@@ -1062,43 +1068,35 @@ static void adsi_load(void)
 	conf = ast_config_load("adsi.conf");
 	if (conf) {
 		x=0;
-		v = ast_variable_browse(conf, "intro");
-		while(v) {
+		for (v = ast_variable_browse(conf, "intro"); v; v = v->next) {
 			if (!strcasecmp(v->name, "alignment"))
 				alignment = str2align(v->value);
 			else if (!strcasecmp(v->name, "greeting")) {
 				if (x < ADSI_MAX_INTRO) {
 					aligns[x] = alignment;
-					strncpy(intro[x], v->value, sizeof(intro[x]) - 1);
-					intro[x][sizeof(intro[x]) - 1] = '\0';
+					ast_copy_string(intro[x], v->value, sizeof(intro[x]));
 					x++;
 				}
 			} else if (!strcasecmp(v->name, "maxretries")) {
 				if (atoi(v->value) > 0)
 					maxretries = atoi(v->value);
 			}
-			v = v->next;
 		}
-		v = ast_variable_browse(conf, "speeddial");
 		if (x)
 			total = x;
 		x = 0;
-		while(v) {
-			char *stringp=NULL;
-			stringp=v->value;
+		for (v = ast_variable_browse(conf, "speeddial"); v; v = v->next) {
+			char *stringp = v->value;
 			name = strsep(&stringp, ",");
 			sname = strsep(&stringp, ",");
 			if (!sname) 
 				sname = name;
 			if (x < ADSI_MAX_SPEED_DIAL) {
-				/* Up to 20 digits */
-				strncpy(speeddial[x][0], v->name, sizeof(speeddial[x][0]) - 1);
-				strncpy(speeddial[x][1], name, 18);
-				strncpy(speeddial[x][2], sname, 7);
+				ast_copy_string(speeddial[x][0], v->name, sizeof(speeddial[x][0]));
+				ast_copy_string(speeddial[x][1], name, 18);
+				ast_copy_string(speeddial[x][2], sname, 7);
 				x++;
 			}
-			v = v->next;
-				
 		}
 		if (x)
 			speeds = x;
@@ -1106,36 +1104,26 @@ static void adsi_load(void)
 	}
 }
 
-int reload(void)
+static int reload(void)
 {
 	adsi_load();
 	return 0;
 }
 
-int load_module(void)
+static int load_module(void)
 {
 	adsi_load();
 	return 0;
 }
 
-int unload_module(void)
+static int unload_module(void)
 {
 	/* Can't unload this once we're loaded */
 	return -1;
 }
 
-char *description(void)
-{
-	return "ADSI Resource";
-}
-
-int usecount(void)
-{
-	/* We should never be unloaded */
-	return 1;
-}
-
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "ADSI Resource",
+		.load = load_module,
+		.unload = unload_module,
+		.reload = reload,
+	       );
